@@ -137,15 +137,28 @@ GFT_MSAN_JOB = JobInfo('googlefuzztest_msan_', 'googlefuzztest', 'memory',
 GFT_UBSAN_JOB = JobInfo('googlefuzztest_ubsan_', 'googlefuzztest', 'undefined',
                         ['googlefuzztest', 'engine_ubsan'])
 
+CENTIPEDE_ASAN_JOB = JobInfo('centipede_asan_', 'centipede', 'address',
+                             ['centipede', 'engine_asan'])
+
+LIBFUZZER_NONE_JOB = JobInfo('libfuzzer_nosanitizer_', 'libfuzzer', 'none',
+                             ['libfuzzer', 'prune'])
+LIBFUZZER_NONE_I386_JOB = JobInfo(
+    'libfuzzer_nosanitizer_i386_',
+    'libfuzzer',
+    'none', ['libfuzzer', 'prune'],
+    architecture='i386')
+
 JOB_MAP = {
     'libfuzzer': {
         'x86_64': {
             'address': LIBFUZZER_ASAN_JOB,
             'memory': LIBFUZZER_MSAN_JOB,
             'undefined': LIBFUZZER_UBSAN_JOB,
+            'none': LIBFUZZER_NONE_JOB,
         },
         'i386': {
             'address': LIBFUZZER_ASAN_I386_JOB,
+            'none': LIBFUZZER_NONE_I386_JOB,
         },
     },
     'afl': {
@@ -169,7 +182,12 @@ JOB_MAP = {
         'x86_64': {
             'address': NO_ENGINE_ASAN_JOB,
         }
-    }
+    },
+    'centipede': {
+        'x86_64': {
+            'address': CENTIPEDE_ASAN_JOB,
+        },
+    },
 }
 
 DEFAULT_ARCHITECTURES = ['x86_64']
@@ -193,7 +211,7 @@ def get_github_url(url):
   response = requests.get(url, auth=(client_id, client_secret))
   if response.status_code != 200:
     logs.log_error(
-        'Failed to get github url: %s' % url, status_code=response.status_code)
+        f'Failed to get github url: {url}.', status_code=response.status_code)
     response.raise_for_status()
 
   return json.loads(response.text)
@@ -279,7 +297,7 @@ def get_jobs_for_project(project, info):
   sanitizers = _process_sanitizers_field(
       info.get('sanitizers', DEFAULT_SANITIZERS))
   if not sanitizers:
-    logs.log_error('Invalid sanitizers field for %s.' % project)
+    logs.log_error(f'Invalid sanitizers field for {project}.')
     return []
 
   engines = info.get('fuzzing_engines', DEFAULT_ENGINES)
@@ -350,7 +368,7 @@ def _add_users_to_bucket(info, client, bucket_name, iam_policy):
     if cc in binding['members']:
       continue
 
-    logs.log('Adding %s to bucket IAM for %s' % (cc, bucket_name))
+    logs.log(f'Adding {cc} to bucket IAM for {bucket_name}.')
     # Add CCs one at a time since the API does not work with invalid or
     # non-Google emails.
     modified_iam_policy = storage.add_single_bucket_iam(
@@ -410,6 +428,11 @@ def add_service_account_to_bucket(client, bucket_name, service_account, role):
   storage.set_bucket_iam_policy(client, bucket_name, iam_policy)
 
 
+def has_maintainer(info):
+  """Return whether or not a project has at least one maintainer."""
+  return info.get('primary_contact') or info.get('auto_ccs')
+
+
 def ccs_from_info(info):
   """Get list of CC's from project info."""
 
@@ -423,10 +446,10 @@ def ccs_from_info(info):
       return field_value
     if isinstance(field_value, str):
       return [field_value]
+    if field_value is None:
+      return []
 
-    raise ProjectSetupError(
-        'Bad value for field {field_name}: {field_value}.'.format(
-            field_name=field_name, field_value=field_value))
+    raise ProjectSetupError(f'Bad value for field {field_name}: {field_value}.')
 
   ccs = []
   ccs.extend(_get_ccs('primary_contact', allow_list=False))
@@ -438,33 +461,35 @@ def ccs_from_info(info):
 
 def update_fuzzer_jobs(fuzzer_entities, job_names):
   """Update fuzzer job mappings."""
-  to_delete = []
+  to_delete = {}
 
-  for job in data_types.Job.query():
-    if not job.environment_string:
-      continue
+  for fuzzer_entity_key in fuzzer_entities:
+    fuzzer_entity = fuzzer_entity_key.get()
 
-    job_environment = job.get_environment()
-    if not utils.string_is_true(job_environment.get('MANAGED', 'False')):
-      continue
+    for job in data_types.Job.query():
+      if not job.environment_string:
+        continue
 
-    if job.name in job_names:
-      continue
+      job_environment = job.get_environment()
+      if not utils.string_is_true(job_environment.get('MANAGED', 'False')):
+        continue
+
+      if job.name in job_names:
+        continue
+
+      logs.log(f'Deleting job {job.name}')
+      to_delete[job.name] = job.key
 
-    logs.log('Deleting job %s' % job.name)
-    to_delete.append(job.key)
-    for fuzzer_entity in fuzzer_entities:
       try:
         fuzzer_entity.jobs.remove(job.name)
       except ValueError:
         pass
 
-  for fuzzer_entity in fuzzer_entities:
     fuzzer_entity.put()
     fuzzer_selection.update_mappings_for_fuzzer(fuzzer_entity)
 
   if to_delete:
-    ndb_utils.delete_multi(to_delete)
+    ndb_utils.delete_multi(to_delete.values())
 
 
 def cleanup_old_projects_settings(project_names):
@@ -473,7 +498,7 @@ def cleanup_old_projects_settings(project_names):
 
   for project in data_types.OssFuzzProject.query():
     if project.name not in project_names:
-      logs.log('Deleting project %s' % project.name)
+      logs.log(f'Deleting project {project.name}.')
       to_delete.append(project.key)
 
   if to_delete:
@@ -609,7 +634,7 @@ class ProjectSetup(object):
 
   def _deployment_bucket_name(self):
     """Deployment bucket name."""
-    return '{project}-deployment'.format(project=utils.get_application_id())
+    return f'{utils.get_application_id()}-deployment'
 
   def _shared_corpus_bucket_name(self):
     """Shared corpus bucket name."""
@@ -637,8 +662,12 @@ class ProjectSetup(object):
 
   def _create_service_accounts_and_buckets(self, project, info):
     """Create per-project service account and buckets."""
-    service_account = service_accounts.get_or_create_service_account(project)
-    service_accounts.set_service_account_roles(service_account)
+    service_account, exists = service_accounts.get_or_create_service_account(
+        project)
+    if not exists:
+      # TODO(ochang): Temporary hack to get around
+      # https://github.com/google/clusterfuzz/issues/2775.
+      service_accounts.set_service_account_roles(service_account)
 
     # Create GCS buckets.
     backup_bucket_name = self._backup_bucket_name(project)
@@ -659,7 +688,7 @@ class ProjectSetup(object):
       add_bucket_iams(info, client, logs_bucket_name, service_account)
       add_bucket_iams(info, client, quarantine_bucket_name, service_account)
     except Exception as e:
-      logs.log_error('Failed to add bucket IAMs for %s: %s' % (project, e))
+      logs.log_error(f'Failed to add bucket IAMs for {project}: {e}.')
 
     # Grant the service account read access to deployment, shared corpus and
     # mutator plugin buckets.
@@ -671,11 +700,13 @@ class ProjectSetup(object):
                                   service_account, OBJECT_VIEWER_IAM_ROLE)
 
     data_bundles = {
-        fuzzer_entity.data_bundle_name
+        fuzzer_entity.get().data_bundle_name
         for fuzzer_entity in six.itervalues(self._fuzzer_entities)
-        if fuzzer_entity.data_bundle_name
     }
     for data_bundle in data_bundles:
+      if not data_bundle:
+        continue
+
       # Workers also need to be able to set up these global bundles.
       data_bundle_bucket_name = data_handler.get_data_bundle_bucket_name(
           data_bundle)
@@ -711,7 +742,7 @@ class ProjectSetup(object):
         # Engine-less jobs are not automatically managed.
         continue
 
-      fuzzer_entity = self._fuzzer_entities.get(template.engine)
+      fuzzer_entity = self._fuzzer_entities.get(template.engine).get()
       if not fuzzer_entity:
         raise ProjectSetupError('Invalid fuzzing engine ' + template.engine)
 
@@ -738,6 +769,7 @@ class ProjectSetup(object):
         if job_name not in fuzzer_entity.jobs and not job.is_external():
           # Enable new job.
           fuzzer_entity.jobs.append(job_name)
+          fuzzer_entity.put()
 
       job.name = job_name
       if self._segregate_projects:
@@ -748,14 +780,27 @@ class ProjectSetup(object):
 
       job.templates = template.cf_job_templates
 
+      # Centipede always uses unsanitized binary as the main fuzz target.
+      if template.engine == 'centipede':
+        build_bucket_path = self._get_build_bucket_path(
+            project, info, template.engine, 'none', template.architecture)
+      else:
+        build_bucket_path = self._get_build_bucket_path(
+            project, info, template.engine, template.memory_tool,
+            template.architecture)
       job.environment_string = JOB_TEMPLATE.format(
           build_type=self._build_type,
-          build_bucket_path=self._get_build_bucket_path(
-              project, info, template.engine, template.memory_tool,
-              template.architecture),
+          build_bucket_path=build_bucket_path,
           engine=template.engine,
           project=project)
 
+      # Centipede requires a separate build of the sanitized binary.
+      if template.engine == 'centipede':
+        extra_build_bucket_path = self._get_build_bucket_path(
+            project, info, template.engine, template.memory_tool,
+            template.architecture)
+        job.environment_string += (
+            f'EXTRA_BUILD_BUCKET_PATH = {extra_build_bucket_path}\n')
       if self._add_revision_mappings:
         revision_vars_url = self._revision_url_template.format(
             project=project,
@@ -763,37 +808,32 @@ class ProjectSetup(object):
                                           template.architecture),
             sanitizer=template.memory_tool)
 
-        job.environment_string += (
-            'REVISION_VARS_URL = {revision_vars_url}\n'.format(
-                revision_vars_url=revision_vars_url))
+        job.environment_string += f'REVISION_VARS_URL = {revision_vars_url}\n'
 
       if logs_bucket_name:
-        job.environment_string += 'FUZZ_LOGS_BUCKET = {logs_bucket}\n'.format(
-            logs_bucket=logs_bucket_name)
+        job.environment_string += f'FUZZ_LOGS_BUCKET = {logs_bucket_name}\n'
 
       if corpus_bucket_name:
-        job.environment_string += 'CORPUS_BUCKET = {corpus_bucket}\n'.format(
-            corpus_bucket=corpus_bucket_name)
+        job.environment_string += f'CORPUS_BUCKET = {corpus_bucket_name}\n'
 
       if quarantine_bucket_name:
         job.environment_string += (
-            'QUARANTINE_BUCKET = {quarantine_bucket}\n'.format(
-                quarantine_bucket=quarantine_bucket_name))
+            f'QUARANTINE_BUCKET = {quarantine_bucket_name}\n')
 
       if backup_bucket_name:
-        job.environment_string += 'BACKUP_BUCKET = {backup_bucket}\n'.format(
-            backup_bucket=backup_bucket_name)
+        job.environment_string += f'BACKUP_BUCKET = {backup_bucket_name}\n'
 
       if self._add_info_labels:
-        job.environment_string += (
-            'AUTOMATIC_LABELS = Proj-{project},Engine-{engine}\n'.format(
-                project=project,
-                engine=template.engine,
-            ))
+        automatic_labels = [f'Proj-{project}', f'Engine-{template.engine}']
+        labels = info.get('labels')
+        if labels and '*' in labels:
+          automatic_labels.extend(labels['*'])
+        automatic_labels = ','.join(automatic_labels)
+        job.environment_string += f'AUTOMATIC_LABELS = {automatic_labels}\n'
 
       help_url = info.get('help_url')
       if help_url:
-        job.environment_string += 'HELP_URL = %s\n' % help_url
+        job.environment_string += f'HELP_URL = {help_url}\n'
 
       if (template.experimental or
           (self._experimental_sanitizers and
@@ -804,16 +844,20 @@ class ProjectSetup(object):
         minimize_job_override = template.minimize_job_override.job_name(
             project, self._config_suffix)
         job.environment_string += (
-            'MINIMIZE_JOB_OVERRIDE = %s\n' % minimize_job_override)
+            f'MINIMIZE_JOB_OVERRIDE = {minimize_job_override}\n')
 
       view_restrictions = info.get('view_restrictions')
       if view_restrictions:
         if view_restrictions in ALLOWED_VIEW_RESTRICTIONS:
           job.environment_string += (
-              'ISSUE_VIEW_RESTRICTIONS = %s\n' % view_restrictions)
+              f'ISSUE_VIEW_RESTRICTIONS = {view_restrictions}\n')
         else:
-          logs.log_error('Invalid view restriction setting %s for project %s.' %
-                         (view_restrictions, project))
+          logs.log_error(
+              f'Invalid view restriction setting {view_restrictions} '
+              f'for project {project}.')
+
+      if not has_maintainer(info):
+        job.environment_string += 'DISABLE_DISCLOSURE = True\n'
 
       selective_unpack = info.get('selective_unpack')
       if selective_unpack:
@@ -823,6 +867,9 @@ class ProjectSetup(object):
       if main_repo:
         job.environment_string += f'MAIN_REPO = {main_repo}\n'
 
+      file_github_issue = info.get('file_github_issue', False)
+      job.environment_string += f'FILE_GITHUB_ISSUE = {file_github_issue}\n'
+
       if (template.engine == 'libfuzzer' and
           template.architecture == 'x86_64' and
           'dataflow' in info.get('fuzzing_engines', DEFAULT_ENGINES)):
@@ -835,7 +882,7 @@ class ProjectSetup(object):
             memory_tool='dataflow',
             architecture=template.architecture)
         job.environment_string += (
-            'DATAFLOW_BUILD_BUCKET_PATH = %s\n' % dataflow_build_bucket_path)
+            f'DATAFLOW_BUILD_BUCKET_PATH = {dataflow_build_bucket_path}\n')
 
       if self._additional_vars:
         additional_vars = {}
@@ -846,9 +893,9 @@ class ProjectSetup(object):
         additional_vars.update(engine_sanitizer_vars)
 
         for key, value in sorted(six.iteritems(additional_vars)):
-          job.environment_string += ('{} = {}\n'.format(
-              key,
-              str(value).encode('unicode-escape').decode('utf-8')))
+          job.environment_string += (
+              f'{key} = {str(value).encode("unicode-escape").decode("utf-8")}\n'
+          )
 
       job.put()
 
@@ -895,7 +942,7 @@ class ProjectSetup(object):
     up."""
     job_names = []
     for project, info in projects:
-      logs.log('Syncing configs for %s.' % project)
+      logs.log(f'Syncing configs for {project}.')
 
       backup_bucket_name = None
       corpus_bucket_name = None
@@ -971,6 +1018,12 @@ class Handler(base_handler.Handler):
       logs.log_error('Failed to get googlefuzztest Fuzzer entity.')
       return
 
+    centipede = data_types.Fuzzer.query(
+        data_types.Fuzzer.name == 'centipede').get()
+    if not centipede:
+      logs.log_error('Failed to get Centipede Fuzzer entity.')
+      return
+
     project_config = local_config.ProjectConfig()
     segregate_projects = project_config.get('segregate_projects')
     project_setup_configs = project_config.get('project_setup')
@@ -978,10 +1031,11 @@ class Handler(base_handler.Handler):
     job_names = set()
 
     fuzzer_entities = {
-        'afl': afl,
-        'honggfuzz': honggfuzz,
-        'googlefuzztest': gft,
-        'libfuzzer': libfuzzer,
+        'afl': afl.key,
+        'centipede': centipede.key,
+        'honggfuzz': honggfuzz.key,
+        'googlefuzztest': gft.key,
+        'libfuzzer': libfuzzer.key,
     }
 
     for setup_config in project_setup_configs:
@@ -1007,6 +1061,7 @@ class Handler(base_handler.Handler):
               'googlefuzztest': bucket_config.get('googlefuzztest'),
               'none': bucket_config.get('no_engine'),
               'dataflow': bucket_config.get('dataflow'),
+              'centipede': bucket_config.get('centipede'),
           },
           fuzzer_entities=fuzzer_entities,
           add_info_labels=setup_config.get('add_info_labels', False),
