@@ -26,6 +26,7 @@ from clusterfuzz._internal.google_cloud_utils import pubsub
 from clusterfuzz._internal.metrics import logs
 from clusterfuzz._internal.system import environment
 from libs.issue_management import issue_tracker_policy
+from libs.issue_management import oss_fuzz_github
 
 NON_CRASH_TYPES = [
     'Data race',
@@ -71,6 +72,13 @@ MEMORY_TOOLS_LABELS = [
 ]
 
 STACKFRAME_LINE_REGEX = re.compile(r'\s*#\d+\s+0x[0-9A-Fa-f]+\s*')
+CHROMIUM_MIRACLEPTR_REGEX = re.compile(r'.*MiraclePtr Status:.+')
+
+MIRACLEPTR_STATUS = {
+    'PROTECTED': 'MiraclePtr-Protected',
+    'MANUAL ANALYSIS REQUIRED': 'MiraclePtr-ManualAnalysisRequired',
+    'NOT PROTECTED': 'MiraclePtr-NotProtected'
+}
 
 
 def platform_substitution(label, testcase, _):
@@ -80,6 +88,10 @@ def platform_substitution(label, testcase, _):
     # ChromeOS fuzzers run on Linux platform, so use correct OS-Chrome for
     # tracking.
     platform = 'Chrome'
+  elif environment.is_ios_job(testcase.job_type):
+    # iOS fuzzers run on macOS platform, so use correct OS-iOS for
+    # tracking.
+    platform = 'iOS'
   elif testcase.platform_id:
     platform = testcase.platform_id.split(':')[0].capitalize()
 
@@ -286,6 +298,26 @@ def notify_issue_update(testcase, status):
               })
       ])
 
+  if status in ('verified', 'wontfix'):
+    logs.log(f'Closing issue {testcase.github_issue_num} '
+             f'in GitHub repo {testcase.github_repo_id}: '
+             f'Testcase {testcase.key.id()} is marked as {status}.')
+    oss_fuzz_github.close_issue(testcase)
+
+
+def check_miracleptr_status(testcase):
+  """Look for MiraclePtr status string and return the appropriate label."""
+  stacktrace = data_handler.get_stacktrace(testcase)
+  for line in stacktrace.split('\n'):
+    if CHROMIUM_MIRACLEPTR_REGEX.match(line):
+      status = line.split(':')[-1].strip()
+      try:
+        return MIRACLEPTR_STATUS[status]
+      except:
+        logs.log_error(f'Unknown MiraclePtr status: {line}')
+        break
+  return None
+
 
 def file_issue(testcase,
                issue_tracker,
@@ -293,7 +325,7 @@ def file_issue(testcase,
                user_email=None,
                additional_ccs=None):
   """File an issue for the given test case."""
-  logs.log('Filing new issue for testcase: %d' % testcase.key.id())
+  logs.log(f'Filing new issue for testcase: {testcase.key.id()}.')
 
   policy = issue_tracker_policy.get(issue_tracker.project)
   is_crash = not utils.sub_string_exists_in(NON_CRASH_TYPES,
@@ -313,15 +345,21 @@ def file_issue(testcase,
     issue.labels.add(policy.label('reproducible'))
 
   # Chromium-specific labels.
-  if issue_tracker.project == 'chromium' and testcase.security_flag:
-    # Add reward labels if this is from an external fuzzer contribution.
-    fuzzer = data_types.Fuzzer.query(
-        data_types.Fuzzer.name == testcase.fuzzer_name).get()
-    if fuzzer and fuzzer.external_contribution:
-      issue.labels.add('reward-topanel')
-      issue.labels.add('External-Fuzzer-Contribution')
+  if issue_tracker.project == 'chromium':
+    if testcase.security_flag:
+      # Add reward labels if this is from an external fuzzer contribution.
+      fuzzer = data_types.Fuzzer.query(
+          data_types.Fuzzer.name == testcase.fuzzer_name).get()
+      if fuzzer and fuzzer.external_contribution:
+        issue.labels.add('reward-topanel')
+        issue.labels.add('External-Fuzzer-Contribution')
 
-    update_issue_impact_labels(testcase, issue)
+      update_issue_impact_labels(testcase, issue)
+
+    # Check for MiraclePtr in stacktrace.
+    miracle_label = check_miracleptr_status(testcase)
+    if miracle_label:
+      issue.labels.add(miracle_label)
 
   # Add additional labels from the job definition and fuzzer.
   additional_labels = data_handler.get_additional_values_for_variable(
@@ -375,7 +413,9 @@ def file_issue(testcase,
       issue_restrictions == 'all' or
       (issue_restrictions == 'security' and testcase.security_flag))
 
-  has_accountable_people = bool(ccs)
+  has_accountable_people = (
+      bool(ccs) and not data_handler.get_value_from_job_definition(
+          testcase.job_type, 'DISABLE_DISCLOSURE', False))
 
   # Check for labels with special logic.
   additional_labels = []
@@ -418,11 +458,31 @@ def file_issue(testcase,
     issue.components.add(policy.unreproducible_component)
 
   issue.reporter = user_email
-  issue.save()
+
+  recovered_exception = None
+  try:
+    issue.save()
+  except Exception as e:
+    if policy.fallback_component:
+      # If a fallback component is set, try clearing the existing components
+      # and filing again.
+      # Also save the exception we recovered from.
+      recovered_exception = e
+      issue.components.clear()
+      issue.components.add(policy.fallback_component)
+
+      if policy.fallback_policy_message:
+        message = policy.fallback_policy_message.replace(
+            '%COMPONENTS%', ' '.join(metadata_components))
+        issue.body += '\n\n' + message
+      issue.save()
+    else:
+      raise
 
   # Update the testcase with this newly created issue.
   testcase.bug_information = str(issue.id)
+  oss_fuzz_github.file_issue(testcase)
   testcase.put()
 
   data_handler.update_group_bug(testcase.group_id)
-  return issue.id
+  return issue.id, recovered_exception
