@@ -23,6 +23,7 @@ except ImportError:
 
 import atexit
 import collections
+import enum
 import os
 import re
 import shutil
@@ -46,8 +47,8 @@ from clusterfuzz._internal.datastore import data_types
 from clusterfuzz._internal.fuzzing import strategy
 from clusterfuzz._internal.metrics import logs
 from clusterfuzz._internal.metrics import profiler
+from clusterfuzz._internal.platforms import android
 from clusterfuzz._internal.system import environment
-from clusterfuzz._internal.system import minijail
 from clusterfuzz._internal.system import new_process
 from clusterfuzz._internal.system import shell
 
@@ -69,7 +70,7 @@ PERSISTENT_EXECUTIONS_OPTION = 'n'
 POSTPROCESSING_TIMEOUT = 30
 
 
-class AflOptionType(object):
+class AflOptionType(enum.Enum):
   ARG = 0
   ENV_VAR = 1
 
@@ -131,7 +132,7 @@ class AflConfig(object):
 
       afl_option = self.LIBFUZZER_TO_AFL_OPTIONS[libfuzzer_name]
       if afl_option.type == AflOptionType.ARG:
-        self.additional_afl_arguments.append('%s%s' % (afl_option.name, value))
+        self.additional_afl_arguments.append(f'{afl_option.name}{value}')
       else:
         assert afl_option.type == AflOptionType.ENV_VAR
         self.additional_env_vars[afl_option.name] = value
@@ -153,7 +154,11 @@ class AflConfig(object):
     if not os.path.exists(default_dict_path):
       return
 
-    self.dict_path = default_dict_path
+    if not environment.is_android():
+      self.dict_path = default_dict_path
+    else:
+      self.dict_path = android.util.get_device_path(default_dict_path)
+
     self.additional_afl_arguments.append(constants.DICT_FLAG + self.dict_path)
 
 
@@ -240,6 +245,28 @@ class AflFuzzOutputDirectory(object):
     return os.path.join(self.instance_directory, 'fuzzer_stats')
 
 
+class AflAndroidFuzzOutputDirectory(AflFuzzOutputDirectory):
+  """Helper class used by AflAndroidRunner to deal with AFL's output directory
+  and help copy contents from device to local.
+  """
+
+  def copy_crash_if_needed(self, testcase_path):
+    """Copy the first crash found by AFL. Before calling super method
+    copy the crashes directory from device to local.
+    """
+    logs.log('Copying crash directory from device to local.')
+
+    # Copy the crashes dir from device to local.
+    local_directory = os.path.join(self.instance_directory, 'crashes')
+    device_directory = android.util.get_device_path(local_directory)
+
+    shell.remove_directory(local_directory, recreate=True)
+    android.adb.copy_remote_directory_to_local(device_directory,
+                                               local_directory)
+
+    super().copy_crash_if_needed(testcase_path)
+
+
 class FuzzingStrategies(object):
   """Helper class used by AflRunner classes to decide what strategy to use
   and to record the decision for StatsGetter to use later."""
@@ -260,6 +287,9 @@ class FuzzingStrategies(object):
   # Probability to disable trimming. (AFL_DISABLE_TRIM=1)
   DISABLE_TRIM_PROB = 0.75
 
+  # Probability to keep long running finds. (AFL_KEEP_TIMEOUTS=1)
+  KEEP_TIMEOUTS_PROB = 0.5
+
   # Probability for increased havoc intensity. (AFL_EXPAND_HAVOC_NOW=1)
   EXPAND_HAVOC_PROB = 0.5
 
@@ -483,26 +513,12 @@ class AflRunnerCommon(object):
     for env_var, value in six.iteritems(config.additional_env_vars):
       environment.set_value(env_var, value)
 
-    self._showmap_output_path = None
+    self.showmap_output_path = os.path.join(fuzzer_utils.get_temp_dir(),
+                                            self.SHOWMAP_FILENAME)
     self.merge_timeout = engine_common.get_merge_timeout(DEFAULT_MERGE_TIMEOUT)
     self.showmap_no_output_logged = False
     self._fuzz_args = []
 
-  @property
-  def showmap_output_path(self):
-    """Returns the showmap output path."""
-    # Initialize _showmap_output_path lazily since MiniJailRunner needs to
-    # execute its __init__ before it can be set.
-    if self._showmap_output_path is None:
-      if environment.get_value('USE_MINIJAIL'):
-        self._showmap_output_path = os.path.join(self.chroot.directory,
-                                                 self.SHOWMAP_FILENAME)
-      else:
-        self._showmap_output_path = os.path.join(fuzzer_utils.get_temp_dir(),
-                                                 self.SHOWMAP_FILENAME)
-
-    return self._showmap_output_path
-
   @property
   def stderr_file_path(self):
     """Returns the file for afl to output stack traces."""
@@ -528,6 +544,43 @@ class AflRunnerCommon(object):
       self._fuzzer_stderr = ''
     return self._fuzzer_stderr
 
+  def set_environment_variables(self):
+    """Sets environment variables needed by afl."""
+    # Tell afl_driver to duplicate stderr to STDERR_FILENAME.
+    # Environment variable names and values that must be set before running afl.
+
+    environment.set_value(constants.FORKSRV_INIT_TMOUT_ENV_VAR,
+                          constants.FORKSERVER_TIMEOUT)
+    environment.set_value(constants.FAST_CAL_ENV_VAR, 1)
+    environment.set_value(constants.IGNORE_UNKNOWN_ENVS_ENV_VAR, 1)
+    environment.set_value(constants.SKIP_CRASHES_ENV_VAR, 1)
+    environment.set_value(constants.BENCH_UNTIL_CRASH_ENV_VAR, 1)
+    environment.set_value(constants.SKIP_CPUFREQ_ENV_VAR, 1)
+
+    stderr_file_path = self.stderr_file_path
+    if environment.is_android():
+      stderr_file_path = android.util.get_device_path(self.stderr_file_path)
+    environment.set_value(constants.STDERR_FILENAME_ENV_VAR, stderr_file_path)
+
+  def get_afl_environment_variables(self):
+    afl_environment_vars = []
+    for env_var in os.environ:
+      if env_var and env_var.startswith('AFL_'):
+        afl_environment_vars.append(env_var + '=' +
+                                    str(environment.get_value(env_var)))
+
+    return afl_environment_vars
+
+  def check_return_code(self, result, additional_return_codes=None):
+    expected_return_codes = [0, 1, -6]
+    if additional_return_codes:
+      expected_return_codes += additional_return_codes
+
+    if result.return_code not in expected_return_codes:
+      logs.log_error(
+          f'AFL target exited with abnormal exit code: {result.return_code}.',
+          output=result.output)
+
   def run_single_testcase(self, testcase_path):
     """Runs a single testcase.
 
@@ -545,28 +598,10 @@ class AflRunnerCommon(object):
     self.afl_setup()
     result = self.run_and_wait(additional_args=[testcase_path])
     print('Running command:', engine_common.get_command_quoted(result.command))
-    if result.return_code not in [0, 1, -6]:
-      logs.log_error(
-          'AFL target exited with abnormal exit code: %s.' % result.return_code,
-          output=result.output)
+    self.check_return_code(result)
 
     return result
 
-  def set_environment_variables(self):
-    """Sets environment variables needed by afl."""
-    # Tell afl_driver to duplicate stderr to STDERR_FILENAME.
-    # Environment variable names and values that must be set before running afl.
-    environment.set_value(constants.FORKSRV_INIT_TMOUT_ENV_VAR,
-                          constants.FORKSERVER_TIMEOUT)
-    environment.set_value(constants.FAST_CAL_ENV_VAR, 1)
-    environment.set_value(constants.IGNORE_UNKNOWN_ENVS_ENV_VAR, 1)
-    environment.set_value(constants.SKIP_CRASHES_ENV_VAR, 1)
-    environment.set_value(constants.SKIP_CPUFREQ_ENV_VAR, 1)
-    environment.set_value(constants.BENCH_UNTIL_CRASH_ENV_VAR, 1)
-    environment.set_value(constants.EXPAND_HAVOC_NOW_VAR, 1)
-    environment.set_value(constants.STDERR_FILENAME_ENV_VAR,
-                          self.stderr_file_path)
-
   def afl_setup(self):
     """Make sure we can run afl. Delete any files that afl_driver needs to
     create and set any environmnet variables it needs.
@@ -656,6 +691,8 @@ class AflRunnerCommon(object):
   def generate_afl_args(self,
                         afl_input=None,
                         afl_output=None,
+                        target_path=None,
+                        additional_args=None,
                         mem_limit=constants.MAX_MEMORY_LIMIT):
     """Generate arguments to pass to Process.run_and_wait.
 
@@ -667,6 +704,10 @@ class AflRunnerCommon(object):
       -o parameter to the afl tool. Defaults to
       self.afl_output.output_directory.
 
+      target_path: Path target binary. Defaults to self.target_path.
+
+      additional_args: Additional AFL arguments
+
       mem_limit: Virtual memory limit afl enforces on target binary, passed as
       -m parameter to the afl tool. Defaults to constants.MAX_MEMORY_LIMIT.
 
@@ -682,17 +723,20 @@ class AflRunnerCommon(object):
     if afl_output is None:
       afl_output = self.afl_output.output_directory
 
+    if target_path is None:
+      target_path = self.target_path
+
     afl_args = [
         constants.INSTANCE_ID_FLAG + constants.DEFAULT_INSTANCE_ID,
         constants.INPUT_FLAG + afl_input, constants.OUTPUT_FLAG + afl_output,
         constants.MEMORY_LIMIT_FLAG + str(mem_limit)
     ]
 
-    afl_args.extend(self.config.additional_afl_arguments)
+    if additional_args is not None:
+      afl_args += additional_args
 
-    afl_args.extend(
-        [self.target_path,
-         str(self.config.num_persistent_executions)])
+    afl_args.extend(self.config.additional_afl_arguments)
+    afl_args.extend([target_path, str(self.config.num_persistent_executions)])
 
     return afl_args
 
@@ -782,14 +826,23 @@ class AflRunnerCommon(object):
           self.strategies.DISABLE_TRIM_PROB):
         environment.set_value(constants.DISABLE_TRIM_ENV_VAR, 1)
 
+      # Randomly keep longer running testcases with new coverage.
+      if engine_common.decide_with_probability(
+          self.strategies.KEEP_TIMEOUTS_PROB):
+        environment.set_value(constants.KEEP_TIMEOUTS_ENV_VAR, 1)
+
       # Randomly enable expanded havoc mutation.
       if engine_common.decide_with_probability(
           self.strategies.EXPAND_HAVOC_PROB):
-        environment.set_value(constants.EXPAND_HAVOC_ENV_VAR, 1)
+        environment.set_value(constants.EXPAND_HAVOC_NOW_VAR, 1)
 
       # Always CMPLOG only new finds.
       environment.set_value(constants.CMPLOG_ONLY_NEW_ENV_VAR, 1)
 
+      # Projects should rather be set up they work effectivly with afl++
+      # though for the time being lets allow bad setups to continue working:
+      environment.set_value(constants.IGNORE_PROBLEMS_ENV_VAR, 1)
+
       # Randomly set new vs. old queue selection mechanism.
       if engine_common.decide_with_probability(
           self.strategies.QUEUE_OLD_STRATEGY_PROB):
@@ -803,12 +856,26 @@ class AflRunnerCommon(object):
       self.set_arg(fuzz_args, constants.CMPLOG_LEVEL_FLAG,
                    rand_cmplog_level(self.strategies))
 
-      # Attempt to start the fuzzer.
-      fuzz_result = self.run_and_wait(
-          additional_args=fuzz_args,
-          timeout=max_total_time,
-          terminate_before_kill=True,
-          terminate_wait_time=self.SIGTERM_WAIT_TIME)
+      if not environment.is_android():
+        # Attempt to start the fuzzer.
+        fuzz_result = self.run_and_wait(
+            additional_args=fuzz_args,
+            timeout=max_total_time,
+            terminate_before_kill=True,
+            terminate_wait_time=self.SIGTERM_WAIT_TIME,
+        )
+      else:
+        android_params = []
+        android_params = self.get_afl_environment_variables()
+        android_params.append(android.util.get_device_path(self.afl_fuzz_path))
+
+        # Attempt to start the fuzzer.
+        fuzz_result = self.run_and_wait(
+            additional_args=android_params + fuzz_args,
+            timeout=max_total_time,
+            terminate_before_kill=True,
+            terminate_wait_time=self.SIGTERM_WAIT_TIME,
+        )
 
       # Reduce max_total_time by the amount of time the last attempt took.
       max_total_time -= fuzz_result.time_executed
@@ -935,7 +1002,11 @@ class AflRunnerCommon(object):
     reproducing testcases."""
 
     if self._afl_output is None:
-      self._afl_output = AflFuzzOutputDirectory()
+      if not environment.is_android():
+        self._afl_output = AflFuzzOutputDirectory()
+      else:
+        self._afl_output = AflAndroidFuzzOutputDirectory()
+
     return self._afl_output
 
   def fuzz(self):
@@ -961,6 +1032,14 @@ class AflRunnerCommon(object):
 
     return self.run_afl_fuzz(self._fuzz_args)
 
+  def get_feature_tuple(self, showmap_output):
+    features = set()
+    for match in re.finditer(self.SHOWMAP_REGEX, showmap_output):
+      d = match.groupdict()
+      features.add((int(d['guard']), int(d['hit_count'])))
+
+    return frozenset(features)
+
   def get_file_features(self, input_file_path, showmap_args):
     """Get the features (edge hit counts) of |input_file_path| using
     afl-showmap."""
@@ -1000,15 +1079,7 @@ class AflRunnerCommon(object):
       return None, True
 
     showmap_output = engine_common.read_data_from_file(self.showmap_output_path)
-
-    features = set()
-    for match in re.finditer(self.SHOWMAP_REGEX, showmap_output):
-      d = match.groupdict()
-      features.add((int(d['guard']), int(d['hit_count'])))
-    logs.log('afl-showmap succeedded for '
-             'file {file_path}: {features} features'.format(
-                 file_path=input_file_path, features=len(features)))
-    return frozenset(features), False
+    return self.get_feature_tuple(showmap_output), False
 
   def merge_corpus(self):
     """Merge new testcases into the input corpus."""
@@ -1020,28 +1091,11 @@ class AflRunnerCommon(object):
     except KeyError:
       pass
     self._executable_path = self.afl_showmap_path
-    # Hack around minijail.
-    showmap_args = self._fuzz_args
-    showmap_args[-1] = '1'
-    # Remove arguments that are just for afl-fuzz.
-    self.remove_arg(showmap_args, constants.DICT_FLAG)
-    self.remove_arg(showmap_args, constants.INPUT_FLAG)
-    self.remove_arg(showmap_args, constants.INSTANCE_ID_FLAG)
-    self.remove_arg(showmap_args, constants.MOPT_FLAG)
-    self.remove_arg(showmap_args, constants.CMPLOG_LEVEL_FLAG)
-    self.remove_arg(showmap_args, constants.QUEUE_OLD_STRATEGY_FLAG)
-    self.remove_arg(showmap_args, constants.SCHEDULER_FLAG)
-    self.remove_arg(showmap_args, constants.CMPLOG_FLAG)
-
-    # Replace -o argument.
-    if environment.get_value('USE_MINIJAIL'):
-      showmap_output_path = '/' + self.SHOWMAP_FILENAME
-    else:
-      showmap_output_path = self.showmap_output_path
-    idx = self.get_arg_index(showmap_args, constants.OUTPUT_FLAG)
-    assert idx != -1
-    self.set_arg(showmap_args, constants.OUTPUT_FLAG, showmap_output_path)
-
+    showmap_args = [
+        f'{constants.OUTPUT_FLAG}{self.showmap_output_path}',
+        f'{constants.MEMORY_LIMIT_FLAG}{constants.MAX_MEMORY_LIMIT}',
+        self.target_path, '-'
+    ]
     input_dir = self.afl_input.input_directory
     corpus = Corpus()
     input_inodes = set()
@@ -1095,19 +1149,6 @@ class AflRunnerCommon(object):
     """Make corpus directories libFuzzer compatible, merge new testcases
     if needed and return the number of new testcases added to corpus.
     """
-    # Remove the input directory binding as it isn't needed for merging and
-    # may actually break merging if it was temporary and gets deleted.
-    if environment.get_value('USE_MINIJAIL'):
-      input_directory = self.afl_input.input_directory
-      input_bindings = [
-          binding for binding in self.chroot.bindings
-          if binding.src_path == input_directory
-      ]
-
-      assert len(input_bindings) == 1
-      input_binding = input_bindings[0]
-      self.chroot.bindings.remove(input_binding)
-
     self.afl_input.restore_if_needed()
     # Number of new units created during fuzzing.
     new_units_generated = self.afl_output.count_new_units(self.afl_output.queue)
@@ -1143,12 +1184,16 @@ class AflRunner(AflRunnerCommon, new_process.UnicodeProcessRunner):
     new_process.ProcessRunner.__init__(self, self.afl_fuzz_path)
 
 
-class MinijailAflRunner(AflRunnerCommon, new_process.UnicodeProcessRunnerMixin,
-                        engine_common.MinijailEngineFuzzerRunner):
-  """Minijail AFL runner."""
+class AflAndroidRunner(AflRunnerCommon, new_process.UnicodeProcessRunner):
+  """Afl Android runner."""
+
+  # Time that we need to remove from timeout command to ensure a clean exit
+  # while fuzzing on device
+  # NOTE: if run_and_wait times out before afl on device process it will return
+  # an error, and clusterfuzz will think there was an error with the fuzzing
+  DEVICE_FUZZING_CLEAN_EXIT_TIME = 10.0
 
   def __init__(self,
-               chroot,
                target_path,
                config,
                testcase_file_path,
@@ -1159,66 +1204,145 @@ class MinijailAflRunner(AflRunnerCommon, new_process.UnicodeProcessRunnerMixin,
     super().__init__(target_path, config, testcase_file_path, input_directory,
                      timeout, afl_tools_path, strategy_dict)
 
-    minijail.MinijailProcessRunner.__init__(self, chroot, self.afl_fuzz_path)
+    new_process.ProcessRunner.__init__(
+        self, executable_path=android.adb.get_adb_path())
+    self.showmap_output_path = os.path.join(
+        android.constants.DEVICE_FUZZING_DIR, self.SHOWMAP_FILENAME)
+    self._showmap_results_dir = os.path.join(environment.get_root_directory(),
+                                             "tmp/showmap_results")
 
-  def _get_or_create_chroot_binding(self, corpus_directory):
-    """Return chroot relative paths for the given corpus directories.
+  def _copy_local_directories_to_device(self, local_directories):
+    """Copies local directories to device."""
+    for local_directory in sorted(set(local_directories)):
+      self._copy_local_directory_to_device(local_directory)
 
-    Args:
-      corpus_directories: A list of host corpus directories.
+  def _copy_local_directory_to_device(self, local_directory):
+    """Copies single local directory to device."""
+    device_directory = android.util.get_device_path(local_directory)
+    android.adb.remove_directory(device_directory, recreate=True)
+    android.adb.copy_local_directory_to_remote(local_directory,
+                                               device_directory)
 
-    Returns:
-      A list of chroot relative paths.
-    """
-    chroot_rel_dir = os.path.relpath(corpus_directory, self.chroot.directory)
-    if not chroot_rel_dir.startswith(os.pardir):
-      # Already in chroot.
-      return '/' + chroot_rel_dir
+  def _copy_directories_from_device(self, local_directories):
+    """Copies directories from device to local."""
+    for local_directory in sorted(set(local_directories)):
+      device_directory = android.util.get_device_path(local_directory)
+      shell.remove_directory(local_directory, recreate=True)
 
-    binding = self.chroot.get_binding(corpus_directory)
-    if binding:
-      return binding.dest_path
+      android.adb.copy_remote_directory_to_local(device_directory,
+                                                 local_directory)
 
-    dest_path = '/' + os.path.basename(corpus_directory)
-    self.chroot.add_binding(
-        minijail.ChrootBinding(corpus_directory, dest_path, True))
-
-    return dest_path
+  def afl_setup(self):
+    android.adb.remove_file(android.util.get_device_path(self.stderr_file_path))
+    super().afl_setup()
 
   def run_single_testcase(self, testcase_path):
-    with self._chroot_testcase(testcase_path) as chroot_testcase_path:
-      return super().run_single_testcase(chroot_testcase_path)
+    """Runs a single testcase.
+    Args:
+      testcase_path: Path to testcase to be run.
+    Returns:
+      A new_process.ProcessResult.
+    """
 
-  def generate_afl_args(self,
-                        afl_input=None,
-                        afl_output=None,
-                        mem_limit=constants.MAX_MEMORY_LIMIT):
-    """Overriden generate_afl_args."""
-    if afl_input:
-      minijail_afl_input = self._get_or_create_chroot_binding(afl_input)
-    else:
-      minijail_afl_input = self._get_or_create_chroot_binding(
-          self.afl_input.input_directory)
+    assert not testcase_path.isdigit(), ('We don\'t want to specify number of'
+                                         ' executions by accident.')
+    self.afl_setup()
+    self._executable_path = android.adb.get_adb_path()
+    self._default_args = (['shell'] + self.get_afl_environment_variables() +
+                          [android.util.get_device_path(self.target_path)])
 
-    if afl_output:
-      minijail_afl_output = self._get_or_create_chroot_binding(afl_output)
-    else:
-      minijail_afl_output = self._get_or_create_chroot_binding(
-          self.afl_output.output_directory)
+    device_target_path = android.util.get_device_path(testcase_path)
+    android.adb.copy_local_file_to_remote(testcase_path, device_target_path)
 
-    return super().generate_afl_args(minijail_afl_input, minijail_afl_output,
-                                     mem_limit)
+    result = self.run_and_wait(additional_args=[device_target_path])
 
-  @property
-  def stderr_file_path(self):
-    """Overriden stderr_file_path."""
-    return os.path.join(self.chroot.directory, STDERR_FILENAME)
+    # Copy error to local.
+    android.adb.copy_remote_file_to_local(
+        android.util.get_device_path(self.stderr_file_path),
+        self.stderr_file_path)
 
-  def set_environment_variables(self):
-    """Overridden set_environment_variables."""
-    super().set_environment_variables()
-    environment.set_value(constants.STDERR_FILENAME_ENV_VAR,
-                          '/' + STDERR_FILENAME)
+    self.check_return_code(result, [134])
+    return result
+
+  def get_file_features(self, input_file_path, showmap_args):
+    """Get the features (edge hit counts) of |input_file_path| using
+    afl-showmap."""
+    # TODO(metzman): Figure out if we should worry about CPU affinity errors
+    # here.
+
+    filename = os.path.basename(input_file_path)
+    intput_file_showmap_results_file = os.path.join(self._showmap_results_dir,
+                                                    filename)
+    showmap_output = engine_common.read_data_from_file(
+        intput_file_showmap_results_file)
+    return self.get_feature_tuple(showmap_output), False
+
+  def fuzz(self):
+    self._executable_path = android.adb.get_adb_path()
+    self._default_args = ['shell']
+
+    self.initial_max_total_time = (
+        get_fuzz_timeout(
+            self.strategies.is_mutations_run, full_timeout=self.timeout) -
+        self.AFL_CLEAN_EXIT_TIME - self.SIGTERM_WAIT_TIME)
+
+    on_device_fuzzing_timeout = self.timeout - int(
+        self.AFL_CLEAN_EXIT_TIME + self.SIGTERM_WAIT_TIME +
+        self.DEVICE_FUZZING_CLEAN_EXIT_TIME)
+
+    self._copy_local_directories_to_device([
+        self.afl_input.input_directory,
+        os.path.dirname(os.path.realpath(self.target_path))
+    ])
+
+    device_output_dir = android.util.get_device_path(
+        self.afl_output.output_directory)
+
+    android.adb.create_directory_if_needed(device_output_dir)
+
+    self._fuzz_args = self.generate_afl_args(
+        afl_input=android.util.get_device_path(self.afl_input.input_directory),
+        afl_output=device_output_dir,
+        additional_args=[
+            constants.FUZZING_TIMEOUT_FLAG + str(on_device_fuzzing_timeout)
+        ],
+        target_path=android.util.get_device_path(self.target_path))
+
+    fuzz_result = self.run_afl_fuzz(self._fuzz_args)
+
+    # Generate file features from fuzzing results.
+    device_script_path = os.path.join(android.constants.DEVICE_FUZZING_DIR,
+                                      "run.sh")
+
+    local_script_path = os.path.join(
+        os.path.abspath(os.path.dirname(__file__)),
+        'create_file_features_showmap.sh')
+
+    android.adb.copy_local_file_to_remote(local_script_path, device_script_path)
+    android.adb.run_shell_command(f'chmod 0777 {device_script_path}', root=True)
+    android.adb.run_shell_command(
+        device_script_path + ' --showmap_path ' + android.util.get_device_path(
+            self.afl_showmap_path) + ' --fuzzer_path ' +
+        android.util.get_device_path(self.target_path) + ' --corpus_path ' +
+        android.util.get_device_path(self.afl_output.queue) + ' --output_path '
+        + android.util.get_device_path(self._showmap_results_dir) +
+        ' --seed_path ' + android.util.get_device_path(
+            self.afl_input.input_directory),
+        root=True,
+        log_output=True)
+
+    # Copy all results from device to local.
+    self._copy_directories_from_device(
+        [self.afl_output.output_directory, self._showmap_results_dir])
+
+    android.adb.copy_remote_file_to_local(
+        android.util.get_device_path(self.stderr_file_path),
+        self.stderr_file_path)
+    return fuzz_result
+
+
+class UnshareAflRunner(new_process.ModifierProcessRunnerMixin, AflRunner):
+  """AFL runner which unshares."""
 
 
 class CorpusElement(object):
@@ -1269,7 +1393,8 @@ def _verify_system_config():
     if not os.path.exists(constants.CORE_PATTERN_FILE_PATH):
       return False
 
-    return open(constants.CORE_PATTERN_FILE_PATH).read().strip() == 'core'
+    with open(constants.CORE_PATTERN_FILE_PATH, 'rb') as f:
+      return f.read().strip() == b'core'
 
   if _check_core_pattern_file():
     return
@@ -1370,7 +1495,7 @@ def get_first_stacktrace(stderr_data):
 
 # TODO(mbarbella): After deleting the non-engine AFL code, remove this
 # function and replace it with a simple check based on the timeout set in
-# AFLEngine.fuzz. Mutations should also be moved to fuzz.
+# Engine.fuzz. Mutations should also be moved to fuzz.
 def get_fuzz_timeout(is_mutations_run, full_timeout=None):
   """Get the maximum amount of time that should be spent fuzzing."""
   fuzz_timeout = full_timeout
@@ -1401,54 +1526,20 @@ def prepare_runner(fuzzer_path,
   """Common initialization code shared by the new pipeline and main."""
   # Set up temp dir.
   engine_common.recreate_directory(fuzzer_utils.get_temp_dir())
-
-  if environment.get_value('USE_MINIJAIL'):
-    # Set up chroot and runner.
-    minijail_chroot = minijail.MinijailChroot(
-        base_dir=fuzzer_utils.get_temp_dir())
-
-    build_dir = environment.get_value('BUILD_DIR')
-
-    # While it's possible for dynamic binaries to run without this, they need to
-    # be accessible for symbolization etc. For simplicity we bind BUILD_DIR to
-    # the same location within the chroot, which leaks the directory structure
-    # of CF but this shouldn't be a big deal.
-    minijail_chroot.add_binding(
-        minijail.ChrootBinding(build_dir, build_dir, False))
-
-    # AFL expects various things in /bin.
-    minijail_chroot.add_binding(minijail.ChrootBinding('/bin', '/bin', False))
-
-    # And /usr/bin.
-    minijail_chroot.add_binding(
-        minijail.ChrootBinding('/usr/bin', '/usr/bin', False))
-
-    # Also bind the build dir to /out to make it easier to hardcode references
-    # to data files.
-    minijail_chroot.add_binding(
-        minijail.ChrootBinding(build_dir, '/out', False))
-
-    # map /proc/self/fd -> /dev/fd
-    os.symlink('/proc/self/fd',
-               os.path.join(minijail_chroot.directory, 'dev', 'fd'))
-
-    runner = MinijailAflRunner(
-        minijail_chroot,
-        fuzzer_path,
-        config,
-        testcase_file_path,
-        input_directory,
-        timeout=timeout,
-        strategy_dict=strategy_dict)
-
+  if environment.get_value('USE_UNSHARE'):
+    runner_class = UnshareAflRunner
+  elif environment.is_android():
+    runner_class = AflAndroidRunner
   else:
-    runner = AflRunner(
-        fuzzer_path,
-        config,
-        testcase_file_path,
-        input_directory,
-        timeout=timeout,
-        strategy_dict=strategy_dict)
+    runner_class = AflRunner
+
+  runner = runner_class(
+      fuzzer_path,
+      config,
+      testcase_file_path,
+      input_directory,
+      timeout=timeout,
+      strategy_dict=strategy_dict)
 
   # Make sure afl won't exit because of bad sanitizer options.
   set_additional_sanitizer_options_for_afl_fuzz()
@@ -1525,12 +1616,10 @@ def main(argv):
   # Execute afl-fuzz on the fuzzing target.
   fuzz_result = runner.fuzz()
 
-  command = fuzz_result.command
-  if environment.get_value('USE_MINIJAIL'):
-    command = engine_common.strip_minijail_command(command,
-                                                   runner.afl_fuzz_path)
   # Print info for the fuzzer logs.
-  print(engine_common.get_log_header(command, fuzz_result.time_executed))
+  print(
+      engine_common.get_log_header(fuzz_result.command,
+                                   fuzz_result.time_executed))
 
   print(fuzz_result.output)
 
@@ -1550,7 +1639,7 @@ def main(argv):
                            runner.fuzzer_stderr, fuzz_result.output)
 
     engine_common.dump_big_query_data(stats_getter.stats, testcase_file_path,
-                                      command)
+                                      fuzz_result.command)
 
   finally:
     print(runner.fuzzer_stderr)
