@@ -70,9 +70,11 @@ class LibFuzzerOptions(engine.FuzzOptions):
     self.extra_env = extra_env
     self.use_dataflow_tracing = use_dataflow_tracing
     self.is_mutations_run = is_mutations_run
+    self.merge_back_new_testcases = True
+    self.analyze_dictionary = True
 
 
-class LibFuzzerEngine(engine.Engine):
+class Engine(engine.Engine):
   """LibFuzzer engine implementation."""
 
   @property
@@ -109,12 +111,23 @@ class LibFuzzerEngine(engine.Engine):
     del build_dir
     arguments = fuzzer.get_arguments(target_path)
     grammar = fuzzer.get_grammar(target_path)
-    strategy_pool = strategy_selection.generate_weighted_strategy_pool(
-        strategy_list=strategy.LIBFUZZER_STRATEGY_LIST,
-        use_generator=True,
-        engine_name=self.name)
+
+    if self.do_strategies:
+      strategy_pool = strategy_selection.generate_weighted_strategy_pool(
+          strategy_list=strategy.LIBFUZZER_STRATEGY_LIST,
+          use_generator=True,
+          engine_name=self.name)
+    else:
+      strategy_pool = strategy_selection.StrategyPool()
+
     strategy_info = libfuzzer.pick_strategies(strategy_pool, target_path,
                                               corpus_dir, arguments, grammar)
+    if (strategy.USE_EXTRA_SANITIZERS_STRATEGY.name in
+        strategy_info.fuzzing_strategies):
+      environment.set_value('USE_EXTRA_SANITIZERS', True)
+      environment.disable_lsan()
+    else:
+      environment.set_value('USE_EXTRA_SANITIZERS', False)
 
     arguments.extend(strategy_info.arguments)
 
@@ -141,7 +154,7 @@ class LibFuzzerEngine(engine.Engine):
     dict_path = fuzzer_utils.extract_argument(
         arguments, constants.DICT_FLAG, remove=False)
     if dict_path and not os.path.exists(dict_path):
-      logs.log_error('Invalid dict %s for %s.' % (dict_path, target_path))
+      logs.log_error(f'Invalid dict {dict_path} for {target_path}.')
       fuzzer_utils.extract_argument(arguments, constants.DICT_FLAG)
 
     # If there's no dict argument, check for %target_binary_name%.dict file.
@@ -228,7 +241,7 @@ class LibFuzzerEngine(engine.Engine):
     # Record the stats to make them easily searchable in stackdriver.
     logs.log('Stats calculated.', stats=stat_overrides)
     if new_units_added:
-      logs.log('New units added to corpus: %d.' % new_units_added)
+      logs.log(f'New units added to corpus: {new_units_added}.')
     else:
       logs.log('No new units found.')
 
@@ -250,9 +263,12 @@ class LibFuzzerEngine(engine.Engine):
     libfuzzer.set_sanitizer_options(target_path, fuzz_options=options)
 
     # Directory to place new units.
-    new_corpus_dir = self._create_temp_corpus_dir('new')
+    if options.merge_back_new_testcases:
+      new_corpus_dir = self._create_temp_corpus_dir('new')
+      corpus_directories = [new_corpus_dir] + options.fuzz_corpus_dirs
+    else:
+      corpus_directories = options.fuzz_corpus_dirs
 
-    corpus_directories = [new_corpus_dir] + options.fuzz_corpus_dirs
     fuzz_result = runner.fuzz(
         corpus_directories,
         fuzz_timeout=max_time,
@@ -265,9 +281,9 @@ class LibFuzzerEngine(engine.Engine):
     dict_error_match = DICT_PARSING_FAILED_REGEX.search(fuzz_result.output)
     if dict_error_match:
       logs.log_error(
-          'Dictionary parsing failed (target={target}, line={line}).'.format(
-              target=project_qualified_fuzzer_name,
-              line=dict_error_match.group(1)),
+          'Dictionary parsing failed '
+          f'(target={project_qualified_fuzzer_name}, '
+          f'line={dict_error_match.group(1)}).',
           engine_output=fuzz_result.output)
     elif (not environment.get_value('USE_MINIJAIL') and
           fuzz_result.return_code == constants.LIBFUZZER_ERROR_EXITCODE):
@@ -275,8 +291,7 @@ class LibFuzzerEngine(engine.Engine):
       # Otherwise: we can assume that a return code of 1 means that libFuzzer
       # itself ran into an error.
       logs.log_error(
-          ENGINE_ERROR_MESSAGE +
-          ' (target={target}).'.format(target=project_qualified_fuzzer_name),
+          ENGINE_ERROR_MESSAGE + f' (target={project_qualified_fuzzer_name}).',
           engine_output=fuzz_result.output)
 
     log_lines = fuzz_result.output.splitlines()
@@ -290,7 +305,8 @@ class LibFuzzerEngine(engine.Engine):
     # If we exited with a non-zero return code with no crash file in output from
     # libFuzzer, this is most likely a startup crash. Use an empty testcase to
     # to store it as a crash.
-    if not crash_testcase_file_path and fuzz_result.return_code:
+    if (not crash_testcase_file_path and
+        fuzz_result.return_code not in constants.NONCRASH_RETURN_CODES):
       crash_testcase_file_path = self._create_empty_testcase_file(
           reproducers_dir)
 
@@ -306,22 +322,23 @@ class LibFuzzerEngine(engine.Engine):
     timeout_limit = fuzzer_utils.extract_argument(
         options.arguments, constants.TIMEOUT_FLAG, remove=False)
 
-    expected_duration = runner.get_max_total_time(max_time)
     actual_duration = int(fuzz_result.time_executed)
-    fuzzing_time_percent = 100 * actual_duration / float(expected_duration)
+    fuzzing_time_percent = 100 * actual_duration / float(max_time)
     parsed_stats.update({
         'timeout_limit': int(timeout_limit),
-        'expected_duration': expected_duration,
+        'expected_duration': int(max_time),
         'actual_duration': actual_duration,
         'fuzzing_time_percent': fuzzing_time_percent,
     })
 
     # Remove fuzzing arguments before merge and dictionary analysis step.
-    merge_arguments = options.arguments[:]
-    libfuzzer.remove_fuzzing_arguments(merge_arguments, is_merge=True)
-    self._merge_new_units(target_path, options.corpus_dir, new_corpus_dir,
-                          options.fuzz_corpus_dirs, merge_arguments,
-                          parsed_stats)
+    non_fuzz_arguments = options.arguments.copy()
+    libfuzzer.remove_fuzzing_arguments(non_fuzz_arguments, is_merge=True)
+
+    if options.merge_back_new_testcases:
+      self._merge_new_units(target_path, options.corpus_dir, new_corpus_dir,
+                            options.fuzz_corpus_dirs, non_fuzz_arguments,
+                            parsed_stats)
 
     fuzz_logs = '\n'.join(log_lines)
     crashes = []
@@ -338,9 +355,10 @@ class LibFuzzerEngine(engine.Engine):
           engine.Crash(crash_testcase_file_path, fuzz_logs, reproduce_arguments,
                        actual_duration))
 
-    libfuzzer.analyze_and_update_recommended_dictionary(
-        runner, project_qualified_fuzzer_name, log_lines, options.corpus_dir,
-        merge_arguments)
+    if options.analyze_dictionary:
+      libfuzzer.analyze_and_update_recommended_dictionary(
+          runner, project_qualified_fuzzer_name, log_lines, options.corpus_dir,
+          non_fuzz_arguments)
 
     return engine.FuzzResult(fuzz_logs, fuzz_result.command, crashes,
                              parsed_stats, fuzz_result.time_executed)
@@ -375,7 +393,8 @@ class LibFuzzerEngine(engine.Engine):
         input_path, timeout=max_time, additional_args=arguments)
 
     if result.timed_out:
-      raise TimeoutError('Reproducing timed out\n' + result.output)
+      logs.log_error('Reproducing timed out.', fuzzer_output=result.output)
+      raise TimeoutError('Reproducing timed out.')
 
     return engine.ReproduceResult(result.command, result.return_code,
                                   result.time_executed, result.output)
@@ -429,7 +448,9 @@ class LibFuzzerEngine(engine.Engine):
     # Adjust the time limit for the time we spent on the first merge step.
     max_time -= result_1.time_executed
     if max_time <= 0:
-      raise TimeoutError('Merging new testcases timed out\n' + result_1.logs)
+      logs.log_error(
+          'Merging new testcases timed out.', fuzzer_output=result_1.logs)
+      raise TimeoutError('Merging new testcases timed out.')
 
     # Step 2. Process the new corpus units as well.
     result_2 = self.minimize_corpus(
@@ -491,10 +512,14 @@ class LibFuzzerEngine(engine.Engine):
         merge_control_file=getattr(self, '_merge_control_file', None))
 
     if result.timed_out:
-      raise TimeoutError('Merging new testcases timed out\n' + result.output)
+      logs.log_error(
+          'Merging new testcases timed out.', fuzzer_output=result.output)
+      raise TimeoutError('Merging new testcases timed out.')
 
     if result.return_code != 0:
-      raise MergeError('Merging new testcases failed: ' + result.output)
+      logs.log_error(
+          'Merging new testcases timed out.', fuzzer_output=result.output)
+      raise MergeError('Merging new testcases failed.')
 
     merge_output = result.output
     merge_stats = stats.parse_stats_from_merge_log(merge_output.splitlines())
@@ -532,7 +557,8 @@ class LibFuzzerEngine(engine.Engine):
         additional_args=arguments)
 
     if result.timed_out:
-      raise TimeoutError('Minimization timed out\n' + result.output)
+      logs.log_error('Minimization timed out.', fuzzer_output=result.output)
+      raise TimeoutError('Minimization timed out.')
 
     return engine.ReproduceResult(result.command, result.return_code,
                                   result.time_executed, result.output)
@@ -565,7 +591,8 @@ class LibFuzzerEngine(engine.Engine):
         additional_args=arguments)
 
     if result.timed_out:
-      raise TimeoutError('Cleanse timed out\n' + result.output)
+      logs.log_error('Cleanse timed out.', fuzzer_output=result.output)
+      raise TimeoutError('Cleanse timed out.')
 
     return engine.ReproduceResult(result.command, result.return_code,
                                   result.time_executed, result.output)
