@@ -15,6 +15,7 @@
 Should not have any dependencies.
 """
 
-SYZ_REPRO = 'syz-crush'
+SYZ_CRUSH = 'syz-crush'
+SYZ_REPRO = 'syz-repro'
 
 CLEAN_EXIT_SECS = 10
