@@ -27,7 +27,8 @@ from clusterfuzz._internal.base import tasks
 from clusterfuzz._internal.base import utils
 from clusterfuzz._internal.bot import testcase_manager
 from clusterfuzz._internal.bot.fuzzers import engine_common
-from clusterfuzz._internal.bot.fuzzers.libFuzzer.engine import LibFuzzerEngine
+from clusterfuzz._internal.bot.fuzzers.libFuzzer import \
+    engine as libfuzzer_engine
 from clusterfuzz._internal.bot.minimizer import basic_minimizers
 from clusterfuzz._internal.bot.minimizer import delta_minimizer
 from clusterfuzz._internal.bot.minimizer import errors as minimizer_errors
@@ -404,6 +405,11 @@ def execute_task(testcase_id, job_type):
     do_libfuzzer_minimization(testcase, testcase_file_path)
     return
 
+  if environment.is_engine_fuzzer_job():
+    # TODO(ochang): More robust check for engine minimization support.
+    _skip_minimization(testcase, 'Engine does not support minimization.')
+    return
+
   max_threads = utils.maximum_parallel_processes_allowed()
 
   # Prepare the test case runner.
@@ -477,10 +483,6 @@ def execute_task(testcase_id, job_type):
     create_additional_tasks(testcase)
     return
 
-  # If we've made it this far, the test case appears to be reproducible. Clear
-  # metadata from previous runs had it been marked as potentially flaky.
-  task_creation.mark_unreproducible_if_flaky(testcase, False)
-
   test_runner.set_test_expectations(testcase.security_flag, flaky_stack,
                                     saved_unsymbolized_crash_state)
 
@@ -1126,7 +1128,7 @@ def run_libfuzzer_engine(tool_name, target_name, arguments, testcase_path,
   if not target_path:
     return engine.ReproduceResult([], 0, 0, '')
 
-  engine_impl = LibFuzzerEngine()
+  engine_impl = libfuzzer_engine.Engine()
   if tool_name == 'minimize':
     func = engine_impl.minimize_testcase
   else:
@@ -1268,8 +1270,6 @@ def do_libfuzzer_minimization(testcase, testcase_file_path):
     handle_unreproducible()
     return
 
-  task_creation.mark_unreproducible_if_flaky(testcase, False)
-
   expected_state = initial_crash_result.get_symbolized_data()
   logs.log('Initial crash state: %s\n' % expected_state.crash_state)
 
