@@ -104,6 +104,13 @@ def _check_fixed_for_custom_binary(testcase, job_type, testcase_file_path):
         message='still crashes on latest custom build')
     return
 
+  if result.unexpected_crash:
+    testcase.set_metadata(
+        'crashes_on_unexpected_state', True, update_testcase=False)
+  else:
+    testcase.delete_metadata(
+        'crashes_on_unexpected_state', update_testcase=False)
+
   # Retry once on another bot to confirm our results and in case this bot is in
   # a bad state which we didn't catch through our usual means.
   if data_handler.is_first_retry_for_task(testcase, reset_after_retry=True):
@@ -120,10 +127,6 @@ def _check_fixed_for_custom_binary(testcase, job_type, testcase_file_path):
 
 def _update_issue_metadata(testcase):
   """Update issue metadata."""
-  if testcase.uploader_email:
-    # Trust the uploader specified metadata.
-    return
-
   metadata = engine_common.get_all_issue_metadata_for_testcase(testcase)
   if not metadata:
     return
@@ -326,6 +329,11 @@ def find_fixed_range(testcase_id, job_type):
                                              state.crash_address, state.frames)
     return
 
+  if result.unexpected_crash:
+    testcase.set_metadata('crashes_on_unexpected_state', True)
+  else:
+    testcase.delete_metadata('crashes_on_unexpected_state')
+
   # Don't burden NFS server with caching these random builds.
   environment.set_value('CACHE_STORE', False)
 
@@ -443,6 +451,15 @@ def execute_task(testcase_id, job_type):
     error_message = 'Unable to recover from bad build'
     data_handler.update_testcase_comment(testcase, data_types.TaskState.ERROR,
                                          error_message)
+  except errors.BuildNotFoundError as e:
+    # If an expected build no longer exists, we can't continue.
+    testcase = data_handler.get_testcase_by_id(testcase_id)
+    testcase.fixed = 'NA'
+    testcase.open = False
+    error_message = f'Build {e.revision} not longer exists'
+    data_handler.clear_progression_pending(testcase)
+    data_handler.update_testcase_comment(testcase, data_types.TaskState.ERROR,
+                                         error_message)
 
   # If there is a fine grained bisection service available, request it. Both
   # regression and fixed ranges are requested once. Regression is also requested
