@@ -16,13 +16,13 @@
 from collections import namedtuple
 from distutils import spawn
 import os
-import random
 import re
 import subprocess
 import time
 
 from clusterfuzz._internal.base import errors
 from clusterfuzz._internal.base import utils
+from clusterfuzz._internal.build_management import overrides
 from clusterfuzz._internal.build_management import revisions
 from clusterfuzz._internal.datastore import data_types
 from clusterfuzz._internal.datastore import ndb_utils
@@ -33,7 +33,6 @@ from clusterfuzz._internal.metrics import logs
 from clusterfuzz._internal.platforms import android
 from clusterfuzz._internal.system import archive
 from clusterfuzz._internal.system import environment
-from clusterfuzz._internal.system import new_process
 from clusterfuzz._internal.system import shell
 
 # The default environment variables for specifying build bucket paths.
@@ -76,6 +75,7 @@ FUZZ_TARGET_ALLOWLISTED_PREFIXES = [
     'afl-fuzz',
     'afl-showmap',
     'afl-tmin',
+    'centipede',
     'honggfuzz',
     'jazzer_agent_deploy.jar',
     'jazzer_driver',
@@ -97,6 +97,17 @@ class BuildManagerException(Exception):
   """Build manager exceptions."""
 
 
+def _normalize_target_name(target_path):
+  """Normalize target path, removing file extensions."""
+  # TODO(ochang): Consolidate this with _internal/bot/fuzzers/utils.py.
+  target_name = os.path.basename(target_path)
+  if '@' in target_name:
+    # GFT target names often have periods in their name.
+    return target_name
+
+  return os.path.splitext(target_name)[0]
+
+
 def _base_build_dir(bucket_path):
   """Get the base directory for a build."""
   job_name = environment.get_value('JOB_NAME')
@@ -152,12 +163,16 @@ def _evict_build(current_build_dir):
   least_recently_used_timestamp = None
 
   for build_directory in os.listdir(builds_directory):
-    absolute_build_directory = os.path.join(builds_directory, build_directory)
+    absolute_build_directory = os.path.abspath(
+        os.path.join(builds_directory, build_directory))
     if not os.path.isdir(absolute_build_directory):
       continue
 
-    if absolute_build_directory == current_build_dir:
-      # Don't evict the build we're trying to extract.
+    if os.path.commonpath(
+        [absolute_build_directory,
+         os.path.abspath(current_build_dir)]) == absolute_build_directory:
+      # Don't evict the build we're trying to extract. This could be a parent
+      # directory of where we're currently extracting to.
       continue
 
     build = BaseBuild(absolute_build_directory)
@@ -509,11 +524,11 @@ class Build(BaseBuild):
       # For fuzzing, pick a random fuzz target so that we only un-archive that
       # particular fuzz target and its dependencies and save disk space.  If we
       # are going to unpack everythng in archive based on
-      # |UNPACK_ALL_FUZZ_TARGETS_AND_FILES| in the job definition, then don't set
-      # a random fuzz target before we've unpacked the build. It won't actually
-      # save us anything in this case and can be really expensive for large
-      # builds (such as Chrome OS). Defer setting it until after the build has
-      # been unpacked.
+      # |UNPACK_ALL_FUZZ_TARGETS_AND_FILES| in the job definition, then don't
+      # set a random fuzz target before we've unpacked the build. It won't
+      # actually save us anything in this case and can be really expensive for
+      # large builds (such as Chrome OS). Defer setting it until after the build
+      # has been unpacked.
       self._pick_fuzz_target(
           self._get_fuzz_targets_from_archive(build_local_archive),
           target_weights)
@@ -574,7 +589,7 @@ class Build(BaseBuild):
     for archive_file in archive.iterator(archive_path):
       if fuzzer_utils.is_fuzz_target_local(archive_file.name,
                                            archive_file.handle):
-        fuzz_target = os.path.splitext(os.path.basename(archive_file.name))[0]
+        fuzz_target = _normalize_target_name(archive_file.name)
         yield fuzz_target
 
   def _get_fuzz_targets_from_dir(self, build_dir):
@@ -583,7 +598,7 @@ class Build(BaseBuild):
     from clusterfuzz._internal.bot.fuzzers import utils as fuzzer_utils
 
     for path in fuzzer_utils.get_fuzz_targets(build_dir):
-      yield os.path.splitext(os.path.basename(path))[0]
+      yield _normalize_target_name(path)
 
   def _pick_fuzz_target(self, fuzz_targets, target_weights):
     """Selects a fuzz target for fuzzing."""
@@ -739,69 +754,32 @@ class RegularBuild(Build):
 class FuchsiaBuild(RegularBuild):
   """Represents a Fuchsia build."""
 
-  SYMBOLIZE_REL_PATH = os.path.join('build', 'zircon', 'prebuilt', 'downloads',
-                                    'symbolize')
-  LLVM_SYMBOLIZER_REL_PATH = os.path.join('build', 'buildtools', 'linux-x64',
-                                          'clang', 'bin', 'llvm-symbolizer')
-  FUCHSIA_BUILD_REL_PATH = os.path.join('build', 'out', 'default')
-  FUCHSIA_DIR_REL_PATH = 'build'
-
   def _pick_fuzz_target(self, fuzz_targets, target_weights):
     """No-op, since Fuchsia builds pick targets later than other build types
     and we aren't ready at the point that this is called by the superclass's
     setup()."""
 
   def _get_fuzz_targets_from_dir(self, build_dir):
-    """Returns a list of fuzz targets in a legacy Fuchsia build. For
-    undercoat-driven Fuchsia builds, a running instance is required to
-    enumerate targets so this is a no-op."""
+    """A running instance is required to enumerate targets so this is a
+    no-op."""
+    return []
 
-    if self.use_undercoat:
-      return []
-
-    # Prevent App Engine import issues.
-    from clusterfuzz._internal.platforms.fuchsia.util.fuzzer import Fuzzer
-    from clusterfuzz._internal.platforms.fuchsia.util.host import Host
-    host = Host.from_dir(os.path.join(build_dir, self.FUCHSIA_BUILD_REL_PATH))
-
-    sanitizer = environment.get_memory_tool_name(
-        environment.get_value('JOB_NAME')).lower()
-    return [
-        str(target[0] + '/' + target[1]) for target in Fuzzer.filter(
-            host.fuzzers, '', sanitizer, example_fuzzers=False)
-    ]
-
-  def _setup_legacy_build(self):
-    """setup() for builds that don't use undercoat."""
+  def setup(self):
+    """Fuchsia build setup."""
     # Prevent App Engine import issues.
     from clusterfuzz._internal.platforms import fuchsia
 
-    # Select a fuzzer, as we skipped doing so in the superclass's setup()
-    _set_random_fuzz_target_for_fuzzing_if_needed(
-        self._get_fuzz_targets_from_dir(self.build_dir), self.target_weights)
+    environment.set_value('FUCHSIA_RESOURCES_DIR', self.build_dir)
 
-    # We set these values here, rather than in initial_qemu_setup, since
-    # SYMBOLIZE_REL_PATH and LLVM_SYMBOLIZER_REL_PATH are properties of the
-    # Build object.
-    symbolize_path = os.path.join(self.build_dir, self.SYMBOLIZE_REL_PATH)
-    os.chmod(symbolize_path, 0o777)
-    llvm_symbolizer_path = os.path.join(self.build_dir,
-                                        self.LLVM_SYMBOLIZER_REL_PATH)
-    os.chmod(llvm_symbolizer_path, 0o777)
+    # Allow superclass's setup() to unpack the build
+    assert environment.get_value('UNPACK_ALL_FUZZ_TARGETS_AND_FILES'), \
+        'Fuchsia does not support partial unpacks'
+    result = super().setup()
+    if not result:
+      return result
 
-    logs.log('Initializing QEMU.')
-
-    # Kill any stale processes that may be left over from previous build.
-    fuchsia.device.stop_qemu()
-
-    fuchsia.device.initial_qemu_setup()
-
-    fuchsia.device.start_qemu()
-
-  def _setup_undercoat_build(self):
-    """setup() for builds that do use undercoat."""
-    # Prevent App Engine import issues.
-    from clusterfuzz._internal.platforms import fuchsia
+    # Verify that undercoat is available in this build
+    fuchsia.undercoat.validate_api_version()
 
     # Kill any stale undercoat instances (currently, this is in fact the only
     # path through which instances are shut down)
@@ -816,50 +794,6 @@ class FuchsiaBuild(RegularBuild):
     _set_random_fuzz_target_for_fuzzing_if_needed(fuzz_targets,
                                                   self.target_weights)
 
-  def setup(self):
-    """Fuchsia build setup."""
-    # Prevent App Engine import issues.
-    from clusterfuzz._internal.platforms import fuchsia
-
-    # Decide per-build whether to use undercoat, based on rollout level
-    rollout_level = environment.get_value('FUCHSIA_UNDERCOAT_ROLLOUT_LEVEL', 0)
-    self.use_undercoat = random.random() < rollout_level
-
-    # Initially, only use undercoat for fuzz tasks
-    if environment.get_value('TASK_NAME') != 'fuzz':
-      self.use_undercoat = False
-
-    # Used by platforms.fuchsia.util
-    environment.set_value(
-        'FUCHSIA_DIR', os.path.join(self.build_dir, self.FUCHSIA_DIR_REL_PATH))
-
-    environment.set_value('FUCHSIA_RESOURCES_DIR', self.build_dir)
-
-    # Allow superclass's setup() to unpack the build
-    assert environment.get_value('UNPACK_ALL_FUZZ_TARGETS_AND_FILES'), \
-        'Fuchsia does not support partial unpacks'
-    result = super().setup()
-    if not result:
-      return result
-
-    # If selected, verify that undercoat is available in this build
-    if self.use_undercoat:
-      try:
-        fuchsia.undercoat.validate_api_version()
-      except (OSError, fuchsia.undercoat.UndercoatError) as e:
-        # Log an error and fall back to the legacy implementation
-        logs.log_warn('Undercoat selected but cannot be used: %s' % e)
-        self.use_undercoat = False
-
-    logs.log('Undercoat enabled: %s' % self.use_undercoat)
-    environment.set_value('FUCHSIA_USE_UNDERCOAT', self.use_undercoat)
-
-    # Finally, delegate to appropriate setup method
-    if self.use_undercoat:
-      self._setup_undercoat_build()
-    else:
-      self._setup_legacy_build()
-
     return True
 
 
@@ -907,35 +841,6 @@ class CuttlefishKernelBuild(RegularBuild):
     return True
 
 
-class AndroidEmulatorBuild(RegularBuild):
-  """Represents an Android Emulator build."""
-
-  def setup(self):
-    """Android Emulator build setup."""
-    self._pre_setup()
-
-    # Download emulator image.
-    if not environment.get_value('ANDROID_EMULATOR_BUCKET_PATH'):
-      logs.log_error('ANDROID_EMULATOR_BUCKET_PATH is not set.')
-      return False
-    archive_src_path = environment.get_value('ANDROID_EMULATOR_BUCKET_PATH')
-    archive_dst_path = os.path.join(self.base_build_dir, 'emulator_bundle.zip')
-    storage.copy_file_from(archive_src_path, archive_dst_path)
-
-    # Extract emulator image.
-    self.emulator_path = os.path.join(self.base_build_dir, 'emulator')
-    shell.remove_directory(self.emulator_path)
-    archive.unpack(archive_dst_path, self.emulator_path)
-    shell.remove_file(archive_dst_path)
-
-    # Stop any stale emulator instances.
-    stop_script_path = os.path.join(self.emulator_path, 'stop')
-    stop_proc = new_process.ProcessRunner(stop_script_path)
-    stop_proc.run_and_wait()
-
-    return super().setup()
-
-
 class SymbolizedBuild(Build):
   """Symbolized build."""
 
@@ -1230,14 +1135,15 @@ def get_primary_bucket_path():
   if release_build_bucket_path:
     return release_build_bucket_path
 
-  fuzz_target_build_bucket_path = environment.get_value(
+  fuzz_target_build_bucket_path = get_bucket_path(
       'FUZZ_TARGET_BUILD_BUCKET_PATH')
+
   if fuzz_target_build_bucket_path:
     fuzz_target = environment.get_value('FUZZ_TARGET')
     if not fuzz_target:
       raise BuildManagerException('FUZZ_TARGET is not defined.')
 
-    return fuzz_target_build_bucket_path.replace('%TARGET%', fuzz_target)
+    return _full_fuzz_target_path(fuzz_target_build_bucket_path, fuzz_target)
 
   raise BuildManagerException(
       'RELEASE_BUILD_BUCKET_PATH or FUZZ_TARGET_BUILD_BUCKET_PATH '
@@ -1281,6 +1187,11 @@ def get_revisions_list(bucket_path, testcase=None):
   return revision_list
 
 
+def _base_fuzz_target_name(target_name):
+  """Get the base fuzz target name "X" from "X@Y"."""
+  return target_name.split('@')[0]
+
+
 def _get_targets_list(bucket_path):
   """Get the target list for a given fuzz target bucket path. This is done by
   reading the targets.list file, which contains a list of the currently active
@@ -1296,7 +1207,12 @@ def _get_targets_list(bucket_path):
   listed_targets = set(
       os.path.basename(path.rstrip('/'))
       for path in storage.list_blobs(bucket_dir_path, recursive=False))
-  return [t for t in targets if t in listed_targets]
+  return [t for t in targets if _base_fuzz_target_name(t) in listed_targets]
+
+
+def _full_fuzz_target_path(bucket_path, fuzz_target):
+  """Get the full fuzz target bucket path."""
+  return bucket_path.replace('%TARGET%', _base_fuzz_target_name(fuzz_target))
 
 
 def _setup_split_targets_build(bucket_path, target_weights, revision=None):
@@ -1312,7 +1228,10 @@ def _setup_split_targets_build(bucket_path, target_weights, revision=None):
     raise BuildManagerException(
         'Failed to choose a fuzz target (path=%s).' % bucket_path)
 
-  fuzz_target_bucket_path = bucket_path.replace('%TARGET%', fuzz_target)
+  if fuzz_target not in targets_list:
+    raise errors.BuildNotFoundError(revision, environment.get_value('JOB_NAME'))
+
+  fuzz_target_bucket_path = _full_fuzz_target_path(bucket_path, fuzz_target)
   if not revision:
     revision = _get_latest_revision([fuzz_target_bucket_path])
 
@@ -1330,6 +1249,11 @@ def _get_latest_revision(bucket_paths):
 
     build_urls.append(BuildUrls(bucket_path=bucket_path, urls_list=urls_list))
 
+  if len(build_urls) == 0:
+    logs.log_error(
+        'Attempted to get latest revision, but no build urls were found.')
+    return None
+
   main_build_urls = build_urls[0]
   other_build_urls = build_urls[1:]
 
@@ -1375,7 +1299,7 @@ def setup_regular_build(revision,
   """Sets up build with a particular revision."""
   if not bucket_path:
     # Bucket path can be customized, otherwise get it from the default env var.
-    bucket_path = environment.get_value('RELEASE_BUILD_BUCKET_PATH')
+    bucket_path = get_bucket_path('RELEASE_BUILD_BUCKET_PATH')
 
   build_urls = get_build_urls_list(bucket_path)
   job_type = environment.get_value('JOB_NAME')
@@ -1397,12 +1321,11 @@ def setup_regular_build(revision,
     build_class = build_setup_host.RemoteRegularBuild
   elif environment.platform() == 'FUCHSIA':
     build_class = FuchsiaBuild
-  elif environment.is_android_emulator():
-    build_class = AndroidEmulatorBuild
   elif (environment.is_android_cuttlefish() and
         environment.is_kernel_fuzzer_job()):
     build_class = CuttlefishKernelBuild
 
+  result = None
   build = build_class(
       base_build_dir,
       revision,
@@ -1410,8 +1333,33 @@ def setup_regular_build(revision,
       target_weights=target_weights,
       build_prefix=build_prefix)
   if build.setup():
-    return build
-  return None
+    result = build
+  else:
+    return None
+
+  # Additional binaries to pull (for fuzzing engines such as Centipede).
+  extra_bucket_path = get_bucket_path('EXTRA_BUILD_BUCKET_PATH')
+  if extra_bucket_path:
+    # Import here as this path is not available in App Engine context.
+    from clusterfuzz._internal.bot.fuzzers import utils as fuzzer_utils
+    extra_build_urls = get_build_urls_list(extra_bucket_path)
+    extra_build_url = revisions.find_build_url(extra_bucket_path,
+                                               extra_build_urls, revision)
+    if not extra_build_url:
+      logs.log_error('Error getting extra build url for job %s (r%d).' %
+                     (job_type, revision))
+      return None
+
+    build = build_class(
+        build.build_dir,  # Store inside the main build.
+        revision,
+        extra_build_url,
+        target_weights=target_weights,
+        build_prefix=fuzzer_utils.EXTRA_BUILD_DIR)
+    if not build.setup():
+      return None
+
+  return result
 
 
 def setup_symbolized_builds(revision):
@@ -1491,13 +1439,10 @@ def setup_custom_binary(target_weights=None):
 
 def setup_production_build(build_type):
   """Sets up build with a particular revision."""
-  # Bail out if there are not stable and beta build urls.
+  # Bail out if there are not extended stable, stable and beta build urls.
   if build_type == 'extended_stable':
     build_bucket_path = environment.get_value(
         'EXTENDED_STABLE_BUILD_BUCKET_PATH')
-    # TODO(yuanjunh): remove it after ES exists.
-    if not build_bucket_path:
-      return None
   elif build_type == 'stable':
     build_bucket_path = environment.get_value('STABLE_BUILD_BUCKET_PATH')
   elif build_type == 'beta':
@@ -1559,8 +1504,9 @@ def setup_build(revision=0, target_weights=None):
   if system_binary:
     return setup_system_binary()
 
-  fuzz_target_build_bucket_path = environment.get_value(
+  fuzz_target_build_bucket_path = get_bucket_path(
       'FUZZ_TARGET_BUILD_BUCKET_PATH')
+
   if fuzz_target_build_bucket_path:
     # Split fuzz target build.
     return _setup_split_targets_build(
@@ -1573,9 +1519,15 @@ def setup_build(revision=0, target_weights=None):
   # If no revision is provided, we default to a trunk build.
   bucket_paths = []
   for env_var in DEFAULT_BUILD_BUCKET_PATH_ENV_VARS:
-    bucket_path = environment.get_value(env_var)
+    bucket_path = get_bucket_path(env_var)
     if bucket_path:
       bucket_paths.append(bucket_path)
+    else:
+      logs.log('Bucket path not found for %s' % env_var)
+
+  if len(bucket_paths) == 0:
+    logs.log_error('Attempted a trunk build, but no bucket paths were found.')
+    return None
 
   return setup_trunk_build(bucket_paths, target_weights=target_weights)
 
@@ -1587,10 +1539,11 @@ def is_custom_binary():
 
 
 def has_production_builds():
-  """Return a bool on if job type has build urls for stable and beta builds."""
-  # TODO(yuanjunh): change it if after ES exists.
+  """Return a bool on if job type has build urls for extended stable, stable and
+  beta builds."""
   return (environment.get_value('STABLE_BUILD_BUCKET_PATH') and
-          environment.get_value('BETA_BUILD_BUCKET_PATH'))
+          environment.get_value('BETA_BUILD_BUCKET_PATH') and
+          environment.get_value('EXTENDED_STABLE_BUILD_BUCKET_PATH'))
 
 
 def has_symbolized_builds():
@@ -1663,3 +1616,11 @@ def check_app_path(app_path='APP_PATH'):
   # APP_PATH.
   return (not environment.get_value('APP_NAME') or
           environment.get_value(app_path))
+
+
+def get_bucket_path(name):
+  """Return build bucket path, applying any set overrides."""
+  bucket_path = environment.get_value(name)
+  bucket_path = overrides.check_and_apply_overrides(
+      bucket_path, overrides.PLATFORM_ID_TO_BUILD_PATH_KEY)
+  return bucket_path
