@@ -28,9 +28,15 @@ from clusterfuzz._internal.tests.test_libs import test_utils
 from handlers.cron import manage_vms
 from handlers.cron.helpers import bot_manager
 
+AUTO_HEALING_POLICY = {
+    'healthCheck': 'global/healthChecks/example-check',
+    'initialDelaySec': 300
+}
+
 INSTANCE_GROUPS = {
     'oss-fuzz-linux-zone2-pre-proj2': {
         'targetSize': 1,
+        'autoHealingPolicies': [AUTO_HEALING_POLICY],
     },
     'oss-fuzz-linux-zone2-pre-proj3': {
         'targetSize': 499,
@@ -180,6 +186,168 @@ INSTANCES = {
     } for i in range(1, 3)],
 }
 
+OSS_FUZZ_CLUSTERS = compute_engine_projects.Project(
+    project_id='clusterfuzz-external',
+    clusters=[
+        compute_engine_projects.Cluster(
+            name='oss-fuzz-linux-zone2-pre',
+            gce_zone='us-east2-a',
+            instance_count=997,
+            instance_template='external-pre-zone2',
+            distribute=True,
+            auto_healing_policy=AUTO_HEALING_POLICY,
+            worker=False,
+            high_end=False),
+        compute_engine_projects.Cluster(
+            name='oss-fuzz-linux-zone3-host',
+            gce_zone='us-central1-d',
+            instance_count=2,
+            instance_template='host-zone3',
+            distribute=False,
+            worker=False,
+            high_end=False),
+        compute_engine_projects.Cluster(
+            name='oss-fuzz-linux-zone3-worker',
+            gce_zone='us-central1-d',
+            instance_count=16,
+            instance_template='worker-zone3',
+            distribute=True,
+            worker=True,
+            high_end=False),
+        compute_engine_projects.Cluster(
+            name='oss-fuzz-linux-zone3-host-high-end',
+            gce_zone='us-central1-d',
+            instance_count=1,
+            instance_template='host-high-end-zone3',
+            distribute=False,
+            worker=False,
+            high_end=True),
+        compute_engine_projects.Cluster(
+            name='oss-fuzz-linux-zone3-worker-high-end',
+            gce_zone='us-central1-d',
+            instance_count=2,
+            instance_template='worker-zone3',
+            distribute=True,
+            worker=True,
+            high_end=True),
+    ],
+    instance_templates=[
+        {
+            'name': 'external-pre-zone2',
+            'description': '{"version": 1}',
+            'properties': {
+                'metadata': {
+                    'items': [],
+                },
+                'disks': [{
+                    'initializeParams': {
+                        'diskSizeGb': 30,
+                    },
+                }],
+                'serviceAccounts': [{
+                    'email':
+                        'email',
+                    'scopes': [
+                        'https://www.googleapis.com/auth/'
+                        'devstorage.full_control',
+                        'https://www.googleapis.com/auth/logging.write',
+                        'https://www.googleapis.com/auth/userinfo.email',
+                        'https://www.googleapis.com/auth/appengine.apis',
+                        'https://www.googleapis.com/auth/prodxmon',
+                        'https://www.googleapis.com/auth/bigquery',
+                    ]
+                }],
+            }
+        },
+        {
+            'name': 'host-zone3',
+            'description': '{"version": 1}',
+            'properties': {
+                'metadata': {
+                    'items': [],
+                },
+                'disks': [{
+                    'initializeParams': {
+                        'diskSizeGb': 30,
+                    },
+                }],
+                'serviceAccounts': [{
+                    'email':
+                        'email',
+                    'scopes': [
+                        'https://www.googleapis.com/auth/'
+                        'devstorage.full_control',
+                        'https://www.googleapis.com/auth/logging.write',
+                        'https://www.googleapis.com/auth/userinfo.email',
+                        'https://www.googleapis.com/auth/appengine.apis',
+                        'https://www.googleapis.com/auth/prodxmon',
+                        'https://www.googleapis.com/auth/bigquery',
+                    ]
+                }],
+            }
+        },
+        {
+            'name': 'worker-zone3',
+            'description': '{"version": 1}',
+            'properties': {
+                'metadata': {
+                    'items': [],
+                },
+                'disks': [{
+                    'initializeParams': {
+                        'diskSizeGb': 30,
+                    },
+                }],
+                'serviceAccounts': [{
+                    'email':
+                        'email',
+                    'scopes': [
+                        'https://www.googleapis.com/auth/'
+                        'devstorage.full_control',
+                        'https://www.googleapis.com/auth/logging.write',
+                        'https://www.googleapis.com/auth/userinfo.email',
+                        'https://www.googleapis.com/auth/prodxmon',
+                    ]
+                }],
+            }
+        },
+        {
+            'name': 'host-high-end-zone3',
+            'description': '{"version": 1}',
+            'properties': {
+                'metadata': {
+                    'items': [],
+                },
+                'disks': [{
+                    'initializeParams': {
+                        'diskSizeGb': 100,
+                    },
+                }],
+                'serviceAccounts': [{
+                    'email':
+                        'email',
+                    'scopes': [
+                        'https://www.googleapis.com/auth/'
+                        'devstorage.full_control',
+                        'https://www.googleapis.com/auth/logging.write',
+                        'https://www.googleapis.com/auth/userinfo.email',
+                        'https://www.googleapis.com/auth/prodxmon',
+                    ]
+                }],
+            }
+        },
+    ],
+    host_worker_assignments=[
+        compute_engine_projects.HostWorkerAssignment(
+            host='oss-fuzz-linux-zone3-host',
+            worker='oss-fuzz-linux-zone3-worker',
+            workers_per_host=8),
+        compute_engine_projects.HostWorkerAssignment(
+            host='oss-fuzz-linux-zone3-host-high-end',
+            worker='oss-fuzz-linux-zone3-worker-high-end',
+            workers_per_host=2),
+    ])
+
 
 def mock_resource(spec):
   """Mock resource."""
@@ -308,166 +476,7 @@ class CronTest(unittest.TestCase):
 
     self.mock.is_oss_fuzz.return_value = True
     self.mock.is_running_on_app_engine.return_value = True
-    self.mock.load_project.return_value = compute_engine_projects.Project(
-        project_id='clusterfuzz-external',
-        clusters=[
-            compute_engine_projects.Cluster(
-                name='oss-fuzz-linux-zone2-pre',
-                gce_zone='us-east2-a',
-                instance_count=997,
-                instance_template='external-pre-zone2',
-                distribute=True,
-                worker=False,
-                high_end=False),
-            compute_engine_projects.Cluster(
-                name='oss-fuzz-linux-zone3-host',
-                gce_zone='us-central1-d',
-                instance_count=2,
-                instance_template='host-zone3',
-                distribute=False,
-                worker=False,
-                high_end=False),
-            compute_engine_projects.Cluster(
-                name='oss-fuzz-linux-zone3-worker',
-                gce_zone='us-central1-d',
-                instance_count=16,
-                instance_template='worker-zone3',
-                distribute=True,
-                worker=True,
-                high_end=False),
-            compute_engine_projects.Cluster(
-                name='oss-fuzz-linux-zone3-host-high-end',
-                gce_zone='us-central1-d',
-                instance_count=1,
-                instance_template='host-high-end-zone3',
-                distribute=False,
-                worker=False,
-                high_end=True),
-            compute_engine_projects.Cluster(
-                name='oss-fuzz-linux-zone3-worker-high-end',
-                gce_zone='us-central1-d',
-                instance_count=2,
-                instance_template='worker-zone3',
-                distribute=True,
-                worker=True,
-                high_end=True),
-        ],
-        instance_templates=[
-            {
-                'name': 'external-pre-zone2',
-                'description': '{"version": 1}',
-                'properties': {
-                    'metadata': {
-                        'items': [],
-                    },
-                    'disks': [{
-                        'initializeParams': {
-                            'diskSizeGb': 30,
-                        },
-                    }],
-                    'serviceAccounts': [{
-                        'email':
-                            'email',
-                        'scopes': [
-                            'https://www.googleapis.com/auth/'
-                            'devstorage.full_control',
-                            'https://www.googleapis.com/auth/logging.write',
-                            'https://www.googleapis.com/auth/userinfo.email',
-                            'https://www.googleapis.com/auth/appengine.apis',
-                            'https://www.googleapis.com/auth/prodxmon',
-                            'https://www.googleapis.com/auth/bigquery',
-                        ]
-                    }],
-                }
-            },
-            {
-                'name': 'host-zone3',
-                'description': '{"version": 1}',
-                'properties': {
-                    'metadata': {
-                        'items': [],
-                    },
-                    'disks': [{
-                        'initializeParams': {
-                            'diskSizeGb': 30,
-                        },
-                    }],
-                    'serviceAccounts': [{
-                        'email':
-                            'email',
-                        'scopes': [
-                            'https://www.googleapis.com/auth/'
-                            'devstorage.full_control',
-                            'https://www.googleapis.com/auth/logging.write',
-                            'https://www.googleapis.com/auth/userinfo.email',
-                            'https://www.googleapis.com/auth/appengine.apis',
-                            'https://www.googleapis.com/auth/prodxmon',
-                            'https://www.googleapis.com/auth/bigquery',
-                        ]
-                    }],
-                }
-            },
-            {
-                'name': 'worker-zone3',
-                'description': '{"version": 1}',
-                'properties': {
-                    'metadata': {
-                        'items': [],
-                    },
-                    'disks': [{
-                        'initializeParams': {
-                            'diskSizeGb': 30,
-                        },
-                    }],
-                    'serviceAccounts': [{
-                        'email':
-                            'email',
-                        'scopes': [
-                            'https://www.googleapis.com/auth/'
-                            'devstorage.full_control',
-                            'https://www.googleapis.com/auth/logging.write',
-                            'https://www.googleapis.com/auth/userinfo.email',
-                            'https://www.googleapis.com/auth/prodxmon',
-                        ]
-                    }],
-                }
-            },
-            {
-                'name': 'host-high-end-zone3',
-                'description': '{"version": 1}',
-                'properties': {
-                    'metadata': {
-                        'items': [],
-                    },
-                    'disks': [{
-                        'initializeParams': {
-                            'diskSizeGb': 100,
-                        },
-                    }],
-                    'serviceAccounts': [{
-                        'email':
-                            'email',
-                        'scopes': [
-                            'https://www.googleapis.com/auth/'
-                            'devstorage.full_control',
-                            'https://www.googleapis.com/auth/logging.write',
-                            'https://www.googleapis.com/auth/userinfo.email',
-                            'https://www.googleapis.com/auth/prodxmon',
-                        ]
-                    }],
-                }
-            },
-        ],
-        host_worker_assignments=[
-            compute_engine_projects.HostWorkerAssignment(
-                host='oss-fuzz-linux-zone3-host',
-                worker='oss-fuzz-linux-zone3-worker',
-                workers_per_host=8),
-            compute_engine_projects.HostWorkerAssignment(
-                host='oss-fuzz-linux-zone3-host-high-end',
-                worker='oss-fuzz-linux-zone3-worker-high-end',
-                workers_per_host=2),
-        ])
+    self.mock.load_project.return_value = OSS_FUZZ_CLUSTERS
 
     data_types.OssFuzzProject(
         id='proj1',
@@ -704,6 +713,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone2-pre-proj1',
             'oss-fuzz-linux-zone2-pre-proj1',
             size=100,
+            auto_healing_policy=AUTO_HEALING_POLICY,
             wait_for_instances=False)
     mock_bot_manager.instance_group(
         'oss-fuzz-linux-zone2-pre-proj1').resize.assert_not_called()
@@ -750,6 +760,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone2-pre-proj4',
             'oss-fuzz-linux-zone2-pre-proj4',
             size=99,
+            auto_healing_policy=AUTO_HEALING_POLICY,
             wait_for_instances=False)
     mock_bot_manager.instance_group(
         'oss-fuzz-linux-zone2-pre-proj4').resize.assert_not_called()
@@ -771,6 +782,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone2-pre-proj5',
             'oss-fuzz-linux-zone2-pre-proj5',
             size=99,
+            auto_healing_policy=AUTO_HEALING_POLICY,
             wait_for_instances=False)
     mock_bot_manager.instance_group(
         'oss-fuzz-linux-zone2-pre-proj5').resize.assert_not_called()
@@ -802,6 +814,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone3-host',
             'oss-fuzz-linux-zone3-host',
             size=2,
+            auto_healing_policy={},
             wait_for_instances=False)
 
     mock_bot_manager.instance_group(
@@ -809,6 +822,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone3-host-high-end',
             'oss-fuzz-linux-zone3-host-high-end',
             size=1,
+            auto_healing_policy={},
             wait_for_instances=False)
 
     # Worker instances: created.
@@ -825,6 +839,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone3-worker-proj1',
             'oss-fuzz-linux-zone3-worker-proj1',
             size=1,
+            auto_healing_policy={},
             wait_for_instances=False)
 
     mock_bot_manager.instance_template(
@@ -840,6 +855,7 @@ class CronTest(unittest.TestCase):
             'oss-fuzz-linux-zone3-worker-proj2',
             'oss-fuzz-linux-zone3-worker-proj2',
             size=4,
+            auto_healing_policy={},
             wait_for_instances=False)
 
     six.assertCountEqual(self, [{
@@ -941,6 +957,12 @@ class CronTest(unittest.TestCase):
 class OssFuzzDistributeCpusTest(unittest.TestCase):
   """Tests OSS-Fuzz CPU distribution."""
 
+  def setUp(self):
+    test_helpers.patch(self, [
+        'clusterfuzz._internal.google_cloud_utils.compute_engine_projects.load_project',
+    ])
+    self.mock.load_project.return_value = OSS_FUZZ_CLUSTERS
+
   def test_equal(self):
     """Tests for each project receiving equal share."""
     projects = [
@@ -1067,6 +1089,12 @@ class OssFuzzDistributeCpusTest(unittest.TestCase):
 class AssignHostWorkerTest(unittest.TestCase):
   """Tests host -> worker assignment."""
 
+  def setUp(self):
+    test_helpers.patch(self, [
+        'clusterfuzz._internal.google_cloud_utils.compute_engine_projects.load_project',
+    ])
+    self.mock.load_project.return_value = OSS_FUZZ_CLUSTERS
+
   def test_assign_keep_existing(self):
     """Test that assignment keeps existing assignments."""
     host_names = ['host']
