@@ -34,6 +34,7 @@ from clusterfuzz._internal.build_management import build_manager
 from clusterfuzz._internal.fuzzing import strategy
 from clusterfuzz._internal.metrics import logs
 from clusterfuzz._internal.platforms import android
+from clusterfuzz._internal.platforms import fuchsia
 from clusterfuzz._internal.system import environment
 from clusterfuzz._internal.system import new_process
 from clusterfuzz._internal.system import process_handler
@@ -97,7 +98,7 @@ class PrepareTest(fake_fs_unittest.TestCase):
 
   def test_prepare(self):
     """Test prepare."""
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare('/corpus_dir', '/path/target', '/path')
     self.assertEqual('/corpus_dir', options.corpus_dir)
     six.assertCountEqual(self, [
@@ -120,13 +121,13 @@ class PrepareTest(fake_fs_unittest.TestCase):
 
   def test_prepare_invalid_dict(self):
     """Test prepare with an invalid dict path."""
-    with open('/path/target.options', 'w') as f:
+    with open('/path/target.options', 'w', encoding='utf-8') as f:
       f.write('[libfuzzer]\n'
               'max_len=31337\n'
               'timeout=11\n'
               'dict=not_exist.dict\n')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare('/corpus_dir', '/path/target', '/path')
     six.assertCountEqual(
         self, ['-max_len=31337', '-timeout=11', '-rss_limit_mb=2560', '-arg1'],
@@ -134,11 +135,11 @@ class PrepareTest(fake_fs_unittest.TestCase):
 
   def test_prepare_auto_add_dict(self):
     """Test prepare automatically adding dict argument."""
-    with open('/path/target.options', 'w') as f:
+    with open('/path/target.options', 'w', encoding='utf-8') as f:
       f.write('[libfuzzer]\n' 'max_len=31337\n' 'timeout=11\n')
     self.fs.create_file('/path/target.dict')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare('/corpus_dir', '/path/target', '/path')
     six.assertCountEqual(self, [
         '-max_len=31337', '-timeout=11', '-rss_limit_mb=2560', '-arg1',
@@ -151,7 +152,7 @@ class FuzzAdditionalProcessingTimeoutTest(unittest.TestCase):
 
   def test_no_mutations(self):
     """Test no mutations."""
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine.LibFuzzerOptions(
         '/corpus',
         arguments=[],
@@ -165,7 +166,7 @@ class FuzzAdditionalProcessingTimeoutTest(unittest.TestCase):
 
   def test_mutations(self):
     """Test with mutations."""
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine.LibFuzzerOptions(
         '/corpus',
         arguments=[],
@@ -239,7 +240,7 @@ class FuzzTest(fake_fs_unittest.TestCase):
 
   def test_fuzz(self):
     """Test fuzz."""
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine.LibFuzzerOptions('/corpus', [
         '-arg=1',
         '-timeout=123',
@@ -248,7 +249,7 @@ class FuzzTest(fake_fs_unittest.TestCase):
         '-use_value_profile=1',
     ], [], ['/corpus'], {}, False, False)
 
-    with open(os.path.join(TEST_DIR, 'crash.txt')) as f:
+    with open(os.path.join(TEST_DIR, 'crash.txt'), encoding='utf-8') as f:
       fuzz_output = f.read()
 
     def mock_fuzz(*args, **kwargs):  # pylint: disable=unused-argument
@@ -271,8 +272,9 @@ class FuzzTest(fake_fs_unittest.TestCase):
       mock_merge_calls.append(self.mock.merge.mock_calls[-1])
       self.assertTrue(len(mock_merge_calls) <= 2)
 
-      merge_output_file = 'merge_step_%d.txt' % len(mock_merge_calls)
-      with open(os.path.join(TEST_DIR, merge_output_file)) as f:
+      merge_output_file = f'merge_step_{len(mock_merge_calls)}.txt'
+      with open(
+          os.path.join(TEST_DIR, merge_output_file), encoding='utf-8') as f:
         merge_output = f.read()
 
       self.fs.create_file('/fuzz-inputs/temp-9001/merge-corpus/A')
@@ -382,6 +384,7 @@ class FuzzTest(fake_fs_unittest.TestCase):
         'strategy_corpus_mutations_radamsa': 0,
         'strategy_corpus_subset': 0,
         'strategy_dataflow_tracing': 0,
+        'strategy_extra_sanitizers': 0,
         'strategy_fork': 0,
         'strategy_mutator_plugin': 0,
         'strategy_mutator_plugin_radamsa': 0,
@@ -487,6 +490,8 @@ class BaseIntegrationTest(unittest.TestCase):
     self.mock.get_merge_timeout.return_value = 10
     self.mock.random_choice.side_effect = mock_random_choice
 
+    self.strategies = []
+
 
 @test_utils.integration
 class IntegrationTests(BaseIntegrationTest):
@@ -513,7 +518,7 @@ class IntegrationTests(BaseIntegrationTest):
   def test_single_testcase_crash(self):
     """Tests libfuzzer with a crashing testcase."""
     testcase_path, _ = setup_testcase_and_corpus('crash', 'empty_corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'test_fuzzer')
     result = engine_impl.reproduce(target_path, testcase_path,
                                    ['-timeout=60', '-rss_limit_mb=2560'], 65)
@@ -529,10 +534,10 @@ class IntegrationTests(BaseIntegrationTest):
   def test_fuzz_no_crash(self):
     """Tests fuzzing (no crash)."""
     self.mock.generate_weighted_strategy_pool.return_value = set_strategy_pool(
-        [strategy.VALUE_PROFILE_STRATEGY])
+        self.strategies + [strategy.VALUE_PROFILE_STRATEGY])
 
     _, corpus_path = setup_testcase_and_corpus('empty', 'corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'test_fuzzer')
     dict_path = target_path + '.dict'
@@ -563,10 +568,10 @@ class IntegrationTests(BaseIntegrationTest):
   def test_fuzz_no_crash_with_old_libfuzzer(self):
     """Tests fuzzing (no crash) with an old version of libFuzzer."""
     self.mock.generate_weighted_strategy_pool.return_value = set_strategy_pool(
-        [strategy.VALUE_PROFILE_STRATEGY])
+        self.strategies + [strategy.VALUE_PROFILE_STRATEGY])
 
     _, corpus_path = setup_testcase_and_corpus('empty', 'corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'test_fuzzer_old')
     dict_path = target_path + '.dict'
@@ -595,8 +600,10 @@ class IntegrationTests(BaseIntegrationTest):
 
   def test_fuzz_crash(self):
     """Tests fuzzing (crash)."""
+    self.mock.generate_weighted_strategy_pool.return_value = set_strategy_pool(
+        self.strategies)
     _, corpus_path = setup_testcase_and_corpus('empty', 'corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR,
                                                  'always_crash_fuzzer')
@@ -624,8 +631,7 @@ class IntegrationTests(BaseIntegrationTest):
         '-timeout=60',
     ], results.crashes[0].reproduce_args)
 
-    self.assertIn('Test unit written to {0}/crash-'.format(self.crash_dir),
-                  results.logs)
+    self.assertIn(f'Test unit written to {self.crash_dir}/crash-', results.logs)
     self.assertIn(
         'ERROR: AddressSanitizer: SEGV on unknown address '
         '0x000000000000', results.logs)
@@ -633,12 +639,12 @@ class IntegrationTests(BaseIntegrationTest):
   def test_fuzz_from_subset(self):
     """Tests fuzzing from corpus subset."""
     self.mock.generate_weighted_strategy_pool.return_value = set_strategy_pool(
-        [strategy.CORPUS_SUBSET_STRATEGY])
+        self.strategies + [strategy.CORPUS_SUBSET_STRATEGY])
 
     _, corpus_path = setup_testcase_and_corpus('empty',
                                                'corpus_with_some_files')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'test_fuzzer')
     dict_path = target_path + '.dict'
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
@@ -661,14 +667,14 @@ class IntegrationTests(BaseIntegrationTest):
     testcase_path, _ = setup_testcase_and_corpus('aaaa', 'empty_corpus')
     minimize_output_path = os.path.join(TEMP_DIR, 'minimized_testcase')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(DATA_DIR,
                                                  'crash_with_A_fuzzer')
     result = engine_impl.minimize_testcase(target_path, [], testcase_path,
                                            minimize_output_path, 120)
     self.assertTrue(result)
     self.assertTrue(os.path.exists(minimize_output_path))
-    with open(minimize_output_path) as f:
+    with open(minimize_output_path, encoding='utf-8') as f:
       result = f.read()
       self.assertEqual('A', result)
 
@@ -677,14 +683,14 @@ class IntegrationTests(BaseIntegrationTest):
     testcase_path, _ = setup_testcase_and_corpus('aaaa', 'empty_corpus')
     cleanse_output_path = os.path.join(TEMP_DIR, 'cleansed_testcase')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(DATA_DIR,
                                                  'crash_with_A_fuzzer')
     result = engine_impl.cleanse(target_path, [], testcase_path,
                                  cleanse_output_path, 120)
     self.assertTrue(result)
     self.assertTrue(os.path.exists(cleanse_output_path))
-    with open(cleanse_output_path) as f:
+    with open(cleanse_output_path, encoding='utf-8') as f:
       result = f.read()
       self.assertFalse(all(c == 'A' for c in result))
 
@@ -708,7 +714,7 @@ class IntegrationTests(BaseIntegrationTest):
     _, corpus_path = setup_testcase_and_corpus('empty',
                                                'corpus_with_some_files')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(DATA_DIR,
                                                  'analyze_dict_fuzzer')
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
@@ -747,7 +753,7 @@ class IntegrationTests(BaseIntegrationTest):
     custom_mutator_print_string = 'CUSTOM MUTATOR\n'
     try:
       target_path = engine_common.find_fuzzer_path(DATA_DIR, fuzz_target_name)
-      engine_impl = engine.LibFuzzerEngine()
+      engine_impl = engine.Engine()
       options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
       results = engine_impl.fuzz(target_path, options, TEMP_DIR, 5)
     finally:
@@ -765,7 +771,7 @@ class IntegrationTests(BaseIntegrationTest):
     fuzz_target_name = 'analyze_dict_fuzzer'
 
     test_helpers.patch(self, [
-        'clusterfuzz._internal.bot.fuzzers.libFuzzer.engine.LibFuzzerEngine.'
+        'clusterfuzz._internal.bot.fuzzers.libFuzzer.engine.Engine.'
         '_create_merge_corpus_dir',
         'clusterfuzz._internal.system.shell.get_directory_file_count',
     ])
@@ -788,12 +794,12 @@ class IntegrationTests(BaseIntegrationTest):
       minimal_unit_path = os.path.join(new_corpus_directory_path,
                                        minimal_unit_hash)
 
-      with open(minimal_unit_path, 'w+') as file_handle:
+      with open(minimal_unit_path, 'w+', encoding='utf-8') as file_handle:
         file_handle.write(minimal_unit_contents)
 
       # Write the nonminimal unit to the corpus directory.
       nonminimal_unit_path = os.path.join(corpus_path, nonminimal_unit_hash)
-      with open(nonminimal_unit_path, 'w+') as file_handle:
+      with open(nonminimal_unit_path, 'w+', encoding='utf-8') as file_handle:
         file_handle.write(nonminimal_unit_contents)
 
       return merge_directory_path
@@ -803,7 +809,7 @@ class IntegrationTests(BaseIntegrationTest):
         mocked_create_merge_directory)
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, fuzz_target_name)
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
     options.arguments.append('-runs=10')
     engine_impl.fuzz(target_path, options, TEMP_DIR, 5)
@@ -828,7 +834,7 @@ class IntegrationTests(BaseIntegrationTest):
                                                'corpus_with_some_files')
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'exit_fuzzer')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
     options.extra_env['EXIT_FUZZER_CODE'] = '1'
 
@@ -854,7 +860,7 @@ class IntegrationTests(BaseIntegrationTest):
                                                'corpus_with_some_files')
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'exit_fuzzer')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
     options.extra_env['EXIT_FUZZER_CODE'] = exit_code
 
@@ -876,7 +882,7 @@ class IntegrationTests(BaseIntegrationTest):
 
     self.mock.log_error.side_effect = mocked_log_error
     _, corpus_path = setup_testcase_and_corpus('empty', 'corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'test_fuzzer')
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
@@ -905,6 +911,31 @@ class UnshareIntegrationTests(IntegrationTests):
     ] + arguments + corpora_or_testcase)
 
 
+@test_utils.integration
+class ExtraSanitizerIntegrationTests(IntegrationTests):
+  """Extra sanitizers integration tests."""
+
+  def setUp(self):
+    super().setUp()
+    os.environ['ASAN_OPTIONS'] = 'detect_leaks=0'
+    os.environ['USE_EXTRA_SANITIZERS'] = 'True'
+    test_helpers.patch(self, [
+        'clusterfuzz._internal.base.utils.is_oss_fuzz',
+    ])
+
+    self.mock.is_oss_fuzz.return_value = True
+    self.strategies = [strategy.USE_EXTRA_SANITIZERS_STRATEGY]
+
+  def compare_arguments(self, target_path, arguments, corpora_or_testcase,
+                        actual):
+    """Compare expected arguments."""
+    self.assertListEqual(actual, [
+        os.path.join(
+            environment.get_value('ROOT_DIR'), 'resources', 'platform', 'linux',
+            'extra_sanitizers'), target_path
+    ] + arguments + corpora_or_testcase)
+
+
 @test_utils.integration
 class MinijailIntegrationTests(IntegrationTests):
   """Minijail integration tests."""
@@ -953,7 +984,7 @@ class MinijailIntegrationTests(IntegrationTests):
                                                'corpus_with_some_files')
 
     target_path = engine_common.find_fuzzer_path(DATA_DIR, 'exit_fuzzer')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
     options.extra_env['EXIT_FUZZER_CODE'] = exit_code
 
@@ -1008,7 +1039,7 @@ class IntegrationTestsFuchsia(BaseIntegrationTest):
 
     _, corpus_path = setup_testcase_and_corpus('aaaa', 'fuchsia_corpus')
     num_files_original = len(os.listdir(corpus_path))
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     options = engine_impl.prepare(corpus_path, 'example-fuzzers/crash_fuzzer',
                                   DATA_DIR)
@@ -1033,7 +1064,7 @@ class IntegrationTestsFuchsia(BaseIntegrationTest):
     build_manager.setup_build()
     testcase_path, _ = setup_testcase_and_corpus('fuchsia_crash',
                                                  'empty_corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     result = engine_impl.reproduce('example-fuzzers/overflow_fuzzer',
                                    testcase_path,
                                    ['-timeout=25', '-rss_limit_mb=2560'], 30)
@@ -1058,7 +1089,7 @@ class IntegrationTestsFuchsia(BaseIntegrationTest):
     testcase_path, _ = setup_testcase_and_corpus('fuchsia_crash',
                                                  'empty_corpus')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     # Check that it's up properly
     results = engine_impl.reproduce('example-fuzzers/overflow_fuzzer',
@@ -1069,15 +1100,13 @@ class IntegrationTestsFuchsia(BaseIntegrationTest):
     # Force termination
     process_handler.terminate_processes_matching_names('qemu-system-x86_64')
 
-    # Try to fuzz against the dead qemu to trigger log dump (and automatic
-    # recovery behavior, when undercoat is disabled)
+    # Try to fuzz against the dead qemu to trigger log dump
     try:
       engine_impl.reproduce('example-fuzzers/overflow_fuzzer', testcase_path,
                             ['-timeout=25', '-rss_limit_mb=2560'], 30)
-    except:
-      # With undercoat, this is expected to dump logs but ultimately fail to run
-      if not environment.get_value('FUCHSIA_USE_UNDERCOAT'):
-        raise
+    except fuchsia.undercoat.UndercoatError:
+      # This is expected to dump logs but ultimately fail to run
+      pass
 
     # Check the logs for syslog presence
     self.assertIn('{{{reset}}}', self.mock.log_warn.call_args[0][0])
@@ -1094,11 +1123,11 @@ class IntegrationTestsFuchsia(BaseIntegrationTest):
                                                  'empty_corpus')
     minimize_output_path = os.path.join(TEMP_DIR, 'output')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     result = engine_impl.minimize_testcase('example-fuzzers/crash_fuzzer',
                                            ['-runs=1000000'], testcase_path,
                                            minimize_output_path, 30)
-    with open(minimize_output_path) as f:
+    with open(minimize_output_path, encoding='utf-8') as f:
       result = f.read()
       self.assertEqual('HI!', result)
 
@@ -1121,8 +1150,8 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
 
     self.crash_dir = TEMP_DIR
     self.adb_path = android.adb.get_adb_path()
-    self.hwasan_options = 'HWASAN_OPTIONS="%s"' % quote(
-        environment.get_value('HWASAN_OPTIONS'))
+    hwasan_options = quote(environment.get_value('HWASAN_OPTIONS'))
+    self.hwasan_options = f'HWASAN_OPTIONS="{hwasan_options}"'
 
   def device_path(self, local_path):
     """Return device path for a local path."""
@@ -1141,7 +1170,7 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
   def test_single_testcase_crash(self):
     """Tests libfuzzer with a crashing testcase."""
     testcase_path, _ = setup_testcase_and_corpus('crash', 'empty_corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'test_fuzzer')
     result = engine_impl.reproduce(target_path, testcase_path,
@@ -1161,10 +1190,10 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
   def test_fuzz_no_crash(self):
     """Tests fuzzing (no crash)."""
     self.mock.generate_weighted_strategy_pool.return_value = set_strategy_pool(
-        [strategy.VALUE_PROFILE_STRATEGY])
+        self.strategies + [strategy.VALUE_PROFILE_STRATEGY])
 
     _, corpus_path = setup_testcase_and_corpus('empty', 'corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'test_fuzzer')
@@ -1202,7 +1231,7 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
   def test_fuzz_crash(self):
     """Tests fuzzing (crash)."""
     _, corpus_path = setup_testcase_and_corpus('empty', 'corpus')
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
 
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'always_crash_fuzzer')
@@ -1236,8 +1265,8 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
     ], results.crashes[0].reproduce_args)
 
     self.assertIn(
-        'Test unit written to {0}/crash-'.format(
-            self.device_path(self.crash_dir)), results.logs)
+        f'Test unit written to {self.device_path(self.crash_dir)}/crash-',
+        results.logs)
     self.assertIn(
         'ERROR: HWAddressSanitizer: SEGV on unknown address '
         '0x000000000000', results.logs)
@@ -1251,7 +1280,7 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
     _, corpus_path = setup_testcase_and_corpus('empty',
                                                'corpus_with_some_files')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'test_fuzzer')
     dict_path = target_path + '.dict'
@@ -1281,14 +1310,14 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
     testcase_path, _ = setup_testcase_and_corpus('aaaa', 'empty_corpus')
     minimize_output_path = os.path.join(TEMP_DIR, 'minimized_testcase')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'crash_with_A_fuzzer')
     result = engine_impl.minimize_testcase(target_path, [], testcase_path,
                                            minimize_output_path, 120)
     self.assertTrue(result)
     self.assertTrue(os.path.exists(minimize_output_path))
-    with open(minimize_output_path) as f:
+    with open(minimize_output_path, encoding='utf-8') as f:
       result = f.read()
       self.assertEqual('A', result)
 
@@ -1297,14 +1326,14 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
     testcase_path, _ = setup_testcase_and_corpus('aaaa', 'empty_corpus')
     cleanse_output_path = os.path.join(TEMP_DIR, 'cleansed_testcase')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'crash_with_A_fuzzer')
     result = engine_impl.cleanse(target_path, [], testcase_path,
                                  cleanse_output_path, 120)
     self.assertTrue(result)
     self.assertTrue(os.path.exists(cleanse_output_path))
-    with open(cleanse_output_path) as f:
+    with open(cleanse_output_path, encoding='utf-8') as f:
       result = f.read()
       self.assertFalse(all(c == 'A' for c in result))
 
@@ -1328,7 +1357,7 @@ class IntegrationTestsAndroid(BaseIntegrationTest, android_helpers.AndroidTest):
     _, corpus_path = setup_testcase_and_corpus('empty',
                                                'corpus_with_some_files')
 
-    engine_impl = engine.LibFuzzerEngine()
+    engine_impl = engine.Engine()
     target_path = engine_common.find_fuzzer_path(ANDROID_DATA_DIR,
                                                  'analyze_dict_fuzzer')
     options = engine_impl.prepare(corpus_path, target_path, DATA_DIR)
