@@ -146,9 +146,9 @@ class StackParser:
     return False
 
   def update_state_on_match(self,
-                            compiled_regex,
-                            line,
-                            state,
+                            compiled_regex: re.Pattern,
+                            line: str,
+                            state: CrashInfo,
                             new_type=None,
                             new_state=None,
                             new_frame_count=None,
@@ -158,7 +158,7 @@ class StackParser:
                             state_from_group=None,
                             address_filter=lambda s: s,
                             type_filter=lambda s: s,
-                            reset=False):
+                            reset=False) -> re.Match or None:
     """Update the specified parts of the state if we have a match."""
 
     match = compiled_regex.match(line)
@@ -197,9 +197,9 @@ class StackParser:
     return match
 
   def add_frame_on_match(self,
-                         compiled_regex,
-                         line,
-                         state,
+                         compiled_regex: re.Pattern,
+                         line: str,
+                         state: CrashInfo,
                          group=0,
                          frame_filter=_filter_stack_frame,
                          demangle=False,
@@ -679,6 +679,11 @@ class StackParser:
             reset=True,
             type_filter=fix_sanitizer_crash_type)
 
+      # Windows formats and surfaces ILL differently from Linux/Posix
+      # Update crash type for consistency.
+      self.update_state_on_match(
+          WINDOWS_SAN_ILL_REGEX, line, state, new_type='Ill', reset=False)
+
       # Overwrite Unknown-crash type with more generic UNKNOWN type.
       if state.crash_type == 'Unknown-crash':
         state.crash_type = 'UNKNOWN'
@@ -758,6 +763,24 @@ class StackParser:
             type_from_group=3,
             type_filter=get_fault_description_for_android_kernel)
 
+        # Kernel Panic
+        self.update_state_on_match(
+            KERNEL_PANIC,
+            line,
+            state,
+            new_type='Kernel failure',
+            reset=True,
+            type_from_group=1,
+            type_filter=filter_kernel_panic_crash_type)
+
+      self.update_state_on_match(
+          KERNEL_BUG,
+          line,
+          state,
+          new_type='Kernel failure',
+          type_from_group=1,
+          type_filter=filter_kasan_crash_type)
+
       # Generic KASan errors.
       if self.update_state_on_match(
           KASAN_CRASH_TYPE_ADDRESS_REGEX,
@@ -767,7 +790,18 @@ class StackParser:
           type_from_group=1,
           address_from_group=4,
           type_filter=filter_kasan_crash_type):
-        state.crash_address = '0x%s' % state.crash_address
+        state.crash_address = f'0x{state.crash_address}'
+
+      # Generic KASan errors with an address range.
+      if self.update_state_on_match(
+          KASAN_CRASH_TYPE_ADDRESS_RANGE_REGEX,
+          line,
+          state,
+          new_type='Kernel failure',
+          type_from_group=1,
+          address_from_group=3,
+          type_filter=filter_kasan_crash_type):
+        state.crash_address = state.crash_address
 
       # Generic KASan errors without an address.
       self.update_state_on_match(
@@ -785,6 +819,36 @@ class StackParser:
           state,
           new_type='Kernel failure\nGeneral-protection-fault')
 
+      # GPU Failure.
+      self.update_state_on_match(
+          GPU_PROCESS_FAILURE,
+          line,
+          state,
+          new_type='GPU failure',
+          new_state='',
+          reset=True)
+
+      # Command injection bugs detected by extra sanitizers.
+      self.update_state_on_match(
+          EXTRA_SANITIZERS_COMMAND_INJECTION_REGEX,
+          line,
+          state,
+          new_type='Command injection')
+
+      # Arbitrary file open detected by extra sanitizers.
+      self.update_state_on_match(
+          EXTRA_SANITIZERS_ARBITRARY_FILE_OPEN_REGEX,
+          line,
+          state,
+          new_type='Arbitrary file open')
+
+      # Arbitrary DNS resolution detected by extra sanitizers.
+      self.update_state_on_match(
+          EXTRA_SANITIZERS_ARBITRARY_DNS,
+          line,
+          state,
+          new_type='Arbitrary DNS resolution')
+
       # For KASan crashes, additional information about a bad access may come
       # from a later line. Update the type and address if this happens.
       update_kasan_crash_details(state, line)
@@ -808,14 +872,11 @@ class StackParser:
       self.update_state_on_check_failure(
           state, line, SECURITY_DCHECK_FAILURE_REGEX, 'Security DCHECK failure')
 
-      # Timeout/OOM detected by libFuzzer.
+      # Timeout/OOM detected by libFuzzer and Centipede.
       if self.detect_ooms_and_hangs:
-        self.update_state_on_match(
-            LIBFUZZER_TIMEOUT_REGEX,
-            line,
-            state,
-            new_type='Timeout',
-            reset=True)
+        for timeout_regex in [LIBFUZZER_TIMEOUT_REGEX, CENTIPEDE_TIMEOUT_REGEX]:
+          self.update_state_on_match(
+              timeout_regex, line, state, new_type='Timeout', reset=True)
         self.update_state_on_match(
             OUT_OF_MEMORY_REGEX,
             line,
@@ -858,12 +919,26 @@ class StackParser:
             address_from_group=1)
 
         if self.update_state_on_match(
+            JAZZER_JAVA_SECURITY_EXCEPTION_REGEX,
+            line,
+            state,
+            new_type='Security exception'):
+          state.found_java_exception = True
+        elif self.update_state_on_match(
             JAZZER_JAVA_EXCEPTION_REGEX,
             line,
             state,
             new_type='Uncaught exception'):
           state.found_java_exception = True
 
+        if self.update_state_on_match(
+            WYCHEPROOF_JAVA_EXCEPTION,
+            line,
+            state,
+            new_type='Wycheproof error',
+            state_from_group=1):
+          state.found_java_exception = True
+
         # Android fatal exceptions.
         if self.update_state_on_match(
             ANDROID_FATAL_EXCEPTION_REGEX,
@@ -926,6 +1001,12 @@ class StackParser:
             state.crash_state = abort_error
           state.frame_count = MAX_CRASH_STATE_FRAMES
 
+        # V8 API errors.
+        v8_error_match = self.update_state_on_match(
+            V8_ERROR_REGEX, line, state, new_type='V8 API error', reset=True)
+        if v8_error_match:
+          state.crash_state = v8_error_match.group(1) + '\n'
+
         # V8 correctness failure errors.
         self.update_state_on_match(
             V8_CORRECTNESS_FAILURE_REGEX,
@@ -1113,7 +1194,10 @@ class StackParser:
 
       if state.found_java_exception:
         if (state.crash_type in [
-            'CHECK failure', 'Fatal Exception', 'Uncaught exception'
+            'CHECK failure',
+            'Fatal Exception',
+            'Uncaught exception',
+            'Security exception',
         ]):
           self.add_frame_on_match(
               JAVA_EXCEPTION_CRASH_STATE_REGEX, line, state, group=1)
@@ -1131,6 +1215,12 @@ class StackParser:
           state.crash_address = '0x%s' % android_kernel_match.group(1)
         continue
 
+      # Android kernel stack frame without address
+      if self.add_frame_on_match(
+          ANDROID_KERNEL_STACK_FRAME_NO_ADDRESS_REGEX, line, state, group=2):
+        state.found_android_kernel_crash = True
+        continue
+
       # V8 correctness fuzzer metadata.
       if self.add_frame_on_match(
           V8_CORRECTNESS_METADATA_REGEX,
@@ -1319,6 +1409,11 @@ def filter_kasan_crash_type(crash_type):
   return 'Kernel failure\n%s' % crash_type.replace(' ', '-').capitalize()
 
 
+def filter_kernel_panic_crash_type(crash_type):
+  """Filter a kernel panic crash type."""
+  return 'Kernel failure\n%s' % crash_type.replace(' ', '-')
+
+
 def update_crash_state_for_stack_overflow_if_needed(state):
   """For stack-overflow bugs, updates crash state based on cycle detected."""
   if state.crash_type != 'Stack-overflow':
@@ -1387,7 +1482,8 @@ def reverse_python_stacktrace(stacktrace):
           python_stacktrace_split = [line]  # Add the "title" of the stacktrace
           break
     else:
-      if '=========' in line:  # Locate beginning of the sanitizer stacktrace
+      # Locate beginning of the sanitizer stacktrace.
+      if '=========' in line or '== ERROR: ' in line:
         break
       python_stacktrace_split.insert(1, line)
 
