@@ -64,6 +64,8 @@ ASSERT_REGEX_GOOGLE = re.compile(GOOGLE_LOG_FATAL_PREFIX +
 ASSERT_REGEX_GLIBC = re.compile(
     r'.*:\s*assertion [`\'"]?(.*?)[`\'"]? failed\.?$', re.IGNORECASE)
 ASSERT_NOT_REACHED_REGEX = re.compile(r'^\s*SHOULD NEVER BE REACHED\s*$')
+CENTIPEDE_TIMEOUT_REGEX = re.compile(
+    r'^========= Timeout of \d+ seconds exceeded; exiting')
 CFI_ERROR_REGEX = re.compile(
     r'(.*): runtime error: control flow integrity check for type (.*) '
     r'failed during (.*vtable address ([xX0-9a-fA-F]+)|.*)')
@@ -91,6 +93,12 @@ CHROME_MAC_STACK_FRAME_REGEX = re.compile(
     r'(\d+)')  # off[dec] (7)
 MSAN_TSAN_REGEX = re.compile(
     r'.*(ThreadSanitizer|MemorySanitizer):\s+(?!ABRT)(?!ILL)([^(:]+)')
+EXTRA_SANITIZERS_COMMAND_INJECTION_REGEX = re.compile(
+    r'===BUG DETECTED: Shell (corruption|injection)===')
+EXTRA_SANITIZERS_ARBITRARY_FILE_OPEN_REGEX = re.compile(
+    r'===BUG DETECTED: Arbitrary file open===')
+EXTRA_SANITIZERS_ARBITRARY_DNS = re.compile(
+    r'===BUG DETECTED: Arbitrary domain name resolution===')
 FATAL_ERROR_GENERIC_FAILURE = re.compile(r'#\s+()(.*)')
 FATAL_ERROR_CHECK_FAILURE = re.compile(
     r'#\s+(Check failed: |RepresentationChangerError: node #\d+:)(.*)')
@@ -103,11 +111,15 @@ GENERIC_SEGV_HANDLER_REGEX = re.compile(
 GOOGLE_CHECK_FAILURE_REGEX = re.compile(GOOGLE_LOG_FATAL_PREFIX +
                                         r'\s*Check failed[:]\s*(.*)')
 GOOGLE_LOG_FATAL_REGEX = re.compile(GOOGLE_LOG_FATAL_PREFIX + r'\s*(.*)')
+GPU_PROCESS_FAILURE = re.compile(r'.*GPU process exited unexpectedly.*')
 HWASAN_ALLOCATION_TAIL_OVERWRITTEN_ADDRESS_REGEX = re.compile(
     r'.*ERROR: HWAddressSanitizer: allocation-tail-overwritten; '
     r'heap object \[([xX0-9a-fA-F]+),.*of size')
+JAZZER_JAVA_SECURITY_EXCEPTION_REGEX = re.compile(
+    '== Java Exception: .*FuzzerSecurityIssue')
 JAZZER_JAVA_EXCEPTION_REGEX = re.compile('== Java Exception: .*')
 JAVA_EXCEPTION_CRASH_STATE_REGEX = re.compile(r'\s*at (.*)\(.*\)')
+KERNEL_BUG = re.compile(r'kernel BUG at (.*)')
 KASAN_ACCESS_TYPE_REGEX = re.compile(r'(Read|Write) of size ([0-9]+)')
 KASAN_ACCESS_TYPE_ADDRESS_REGEX = re.compile(
     r'(Read|Write) of size ([0-9]+) at (addr|address) ([a-f0-9]+)')
@@ -118,6 +130,7 @@ KASAN_CRASH_TYPE_ADDRESS_RANGE_REGEX = re.compile(
 KASAN_CRASH_TYPE_FUNCTION_REGEX = re.compile(
     r'BUG: KASAN: (.*) (in|on).* ([\w]+)\+([\w]+)\/([\w]+)')
 KASAN_GPF_REGEX = re.compile(r'general protection fault:.*KASAN')
+KERNEL_PANIC = re.compile(r'Kernel panic - not syncing: (.*)')
 LIBFUZZER_DEADLY_SIGNAL_REGEX = re.compile(
     r'.*ERROR:\s*libFuzzer:\s*deadly signal')
 LIBFUZZER_FUZZ_TARGET_EXITED_REGEX = re.compile(
@@ -152,13 +165,15 @@ OUT_OF_MEMORY_REGEX = re.compile(r'.*(?:%s).*' % '|'.join([
     r'Sanitizer: calloc-overflow',
     r'Sanitizer: calloc parameters overflow',
     r'Sanitizer: requested allocation size.*exceeds maximum supported size',
+    r'Sanitizer: out of memory',
     r'TerminateBecauseOutOfMemory',
     r'allocator is out of memory trying to allocate',
     r'blinkGCOutOfMemory',
     r'couldnt allocate.*Out of memory',
     r'libFuzzer: out-of-memory \(',
     r'rss limit exhausted',
-    r'in rust_oom'
+    r'in rust_oom',
+    r'Failure description: out-of-memory',  # Centipede.
 ]))
 RUNTIME_ERROR_REGEX = re.compile(r'#\s*Runtime error in (.*)')
 RUNTIME_ERROR_LINE_REGEX = re.compile(r'#\s*Runtime error in (.*), line [0-9]+')
@@ -262,6 +277,7 @@ V8_ABORT_METADATA_REGEX = re.compile(r'(.*) \[(.*):\d+\]$')
 V8_CORRECTNESS_FAILURE_REGEX = re.compile(r'#\s*V8 correctness failure')
 V8_CORRECTNESS_METADATA_REGEX = re.compile(
     r'#\s*V8 correctness ((configs|sources|suppression): .*)')
+V8_ERROR_REGEX = re.compile(r'\s*\[[^\]]*\] V8 error: (.+)\.$')
 WINDOWS_CDB_STACK_FRAME_REGEX = re.compile(
     r'([0-9a-zA-Z`]+) '  # Child EBP or SP; remove ` if needed (1)
     r'([0-9a-zA-Z`]+) '  # RetAddr; remove ` if needed (2)
@@ -275,6 +291,10 @@ WINDOWS_CDB_CRASH_TYPE_REGEX = re.compile(
     r'.*DEFAULT_BUCKET_ID[ ]*[:][ ]*([a-zA-Z_]+)')
 WINDOWS_CDB_STACK_OVERFLOW_REGEX = re.compile(
     r'.*ExceptionCode: .*\(Stack overflow\).*')
+WINDOWS_SAN_ILL_REGEX = re.compile(r'.*[a-zA-Z]+Sanitizer: illegal-instruction')
+
+WYCHEPROOF_JAVA_EXCEPTION = re.compile(
+    r'.*\) (.*\(com\.google\.security\.wycheproof\.[a-zA-z0-9]*\))')
 
 # Golang specific regular expressions.
 GOLANG_DIVISION_BY_ZERO_REGEX = re.compile(
@@ -301,7 +321,7 @@ GOLANG_CRASH_TYPES_MAP = [
 GOLANG_FATAL_ERROR_REGEX = re.compile(r'^fatal error: (.*)')
 
 GOLANG_STACK_FRAME_FUNCTION_REGEX = re.compile(
-    r'^([0-9a-zA-Z\.\-\_\\\/\(\)\*]+)\([x0-9a-f\s,\.]*\)$')
+    r'^([0-9a-zA-Z\.\-\_\\\/\(\)\*]+)\([x0-9a-f\s,\.{}]*\)$')
 
 # Python specific regular expressions.
 PYTHON_UNHANDLED_EXCEPTION = re.compile(
@@ -367,7 +387,6 @@ STACK_FRAME_IGNORE_REGEXES = [
     r'^SignalAction',
     r'^SignalHandler',
     r'^TestOneProtoInput',
-    r'^V8_Fatal',
     r'^WTF::',
     r'^WTFCrash',
     r'^X11Error',
@@ -419,6 +438,7 @@ STACK_FRAME_IGNORE_REGEXES = [
     r'^calloc',
     r'^check_memory_region',
     r'^common_exit',
+    r'^core::fmt::write',
     r'^delete',
     r'^demangling_terminate_handler',
     r'^dump_backtrace',
@@ -438,18 +458,24 @@ STACK_FRAME_IGNORE_REGEXES = [
     r'^new',
     r'^object_err',
     r'^operator',
+    r'^panic_abort::',
     r'^print_trailer',
     r'^realloc',
     r'^rust_begin_unwind',
     r'^rust_fuzzer_test_input',
     r'^rust_oom',
+    r'^rust_panic',
     r'^scanf',
     r'^show_stack',
     r'^std::__terminate',
+    r'^std::io::Write::write_fmt',
     r'^std::panic',
     r'^std::process::abort',
     r'^std::sys::unix::abort',
+    r'^std::sys_common::backtrace',
+    r'^__rust_start_panic',
     r'^__scrt_common_main_seh',
+    r'^libgcc_s.so.*',
 
     # Functions names (contains).
     r'.*ASAN_OnSIGSEGV',
@@ -510,6 +536,8 @@ STACK_FRAME_IGNORE_REGEXES = [
     r'.*/vctools/crt/',
     r'.*/win_toolchain/',
     r'.*libc\+\+/',
+    # Clusterfuzz file paths on Windows to ignore.
+    r'c:/clusterfuzz/bot/build',
 
     # Wrappers from honggfuzz/libhfuzz/memorycmp.c.
     r'.*/memorycmp\.c',
@@ -520,6 +548,7 @@ STACK_FRAME_IGNORE_REGEXES = [
     r'.*Inline Function @',
     r'^<unknown>$',
     r'^\[vdso\]$',
+    r'^linux-gate.so.*$',
 
     # Golang specific frames to ignore.
     r'^panic$',
@@ -541,6 +570,15 @@ STACK_FRAME_IGNORE_REGEXES = [
 
     # googlefuzztest specific.
     r'.*fuzztest::internal::',
+
+    # V8 specific.
+    r'^V8_Fatal',
+    # Ignore error-throwing frames, the bug is in the caller.
+    r'^blink::ReportV8FatalError',
+    r'^v8::api_internal::ToLocalEmpty',
+
+    # google3 specific stack frame ignores.
+    r'^absl::log_internal::',
 ]
 
 STACK_FRAME_IGNORE_REGEXES_IF_SYMBOLIZED = [
@@ -552,12 +590,15 @@ STACK_FRAME_IGNORE_REGEXES_IF_SYMBOLIZED = [
 ]
 
 IGNORE_CRASH_TYPES_FOR_ABRT_BREAKPOINT_AND_ILLS = [
+    'Arbitrary file open',
     'ASSERT',
     'CHECK failure',
+    'Command injection',
     'DCHECK failure',
     'Fatal error',
     'Security CHECK failure',
     'Security DCHECK failure',
+    'V8 API error',
 ]
 
 STATE_STOP_MARKERS = [
